"""A bunch of classes for producing klasses (instances that are used
to keep information about the classes they represent)"""
import utils, string, os

try:
    from Globals import package_home
    ROOT_DIR = package_home(globals())
except:
    ROOT_DIR = ''


class BasicKlass:
    """A basic klass"""

    _properties = ({'id': 'id',             'type': 'string', 'default': 'id'},
                   {'id': 'bases',          'type': 'bases',  'default': []},
                   {'id': 'subklasses',     'type': 'list',   'default': []},
                   {'id': 'methods',        'type': 'list',   'default': []},
                   {'id': 'properties',     'type': 'list',   'default': []},
                   {'id': 'default_indent', 'type': 'string', 'default': '    '},
                   {'id': 'license',        'type': 'string', 'default': ''},
                   {'id': 'docstring',      'type': 'string', 'default': "'''doc string'''"},
                   {'id': 'meta_type',      'type': 'string', 'default': ''},
                   {'id': 'icon_path',      'type': 'string', 'default': ''},
                   {'id': 'misc_headers',   'type': 'int',    'default': 0},
                   {'id': 'author',         'type': 'string', 'default': 'Foo Bar'},
                   {'id': 'author_email',   'type': 'string', 'default': 'foo@bar.com'},
                   {'id': 'authored_year',  'type': 'string', 'default': ''})


    def __init__(self, **kw):
        """Initialize an instance of the class"""

        for property in self._properties:
            if property['id'] in kw.keys():
                setattr(self, property['id'], kw[property['id']])
            else:
                if property['type'] == 'list':
                    default = list(property['default'])
                else:
                    default = property['default']
                setattr(self, property['id'], default)

        self.subklasses = []
        

    def add_subklass(self, klass):
        """Add a subklass"""

        if not utils.find_entry(self.list_subklasses(), klass.id):
            self.subklasses.append(klass)
        else:
            raise "DuplicateSubklass", "There already exists a " + \
                  "subklass with that 'id'"
        return klass


    def list_subklasses(self):
        """Return a list of all subklasses"""
        return self.subklasses


    def remove_subklass(self, subklass_id):
        """Remove the subklass with the id 'subklass_id'"""
        instance = utils.find_entry(self.list_subklasses(), subklass_id)
        if instance: self.subklasses.remove(instance)


    def get_subklass(self, subklass_id):
        """Return the subklass with id 'subklass_id'"""
        return utils.find_entry(self.list_subklasses(), subklass_id)


class MethodKlass(BasicKlass):
    """A class that provides methods for manipulating methods"""

    def add_method(self, method):
        """Add a method"""
        if not utils.find_entry(self.list_methods(), method.id):
            self.methods.append(method)
        else:
            raise "DuplicateMethod", "There already exsists a " + \
                  "method with that 'id'"
        return method

        
    def list_methods(self):
        """Return a list methods"""
        return self.methods

    
    def remove_method(self, method_id):
        """Remove the method with the id 'method_id'"""
        instance = utils.find_entry(self.list_methods(), method_id)
        if instance: self.methods.remove(instance)

  
    def get_method(self, method_id):
        """Return the method with id 'method_id'"""
        return utils.find_entry(self.list_methods(), method_id)



class MethodKlassRendrer(MethodKlass):
    """Adds methods for rendering"""

    def _render_methods(self, indent=''):
        """Render all methods in this class"""
        methods = ""
        for method in self.list_methods():
            methods = methods + method.render_self(indent) + "\n"
        return methods 



class PropertyKlass(BasicKlass):
    """A class that provides methods for manipulating properties"""

    def add_property(self, property):
        """Add a property"""
        if not utils.find_entry(self.list_properties(), property.id):
            self.properties.append(property)
        else:
            raise "DuplicateProperty", "There already exsists a " + \
                  "property with that 'id'"
        return property

        
    def list_properties(self):
        """Return a list of all properties"""
        return self.properties

    
    def remove_property(self, property_id):
        """Remove the property with id 'property_id'"""
        instance = utils.find_entry(self.list_properties(), property_id)
        if instance: self.properties.remove(instance)


    def get_property(self, property_id):
        """Get property with id 'property_id'"""
        return utils.find_entry(self.list_properties(), property_id)


                                    
class PropertyKlassRendrer(PropertyKlass):
    """Adds methods for rendering"""

    def _render_properties(self, indent=''):
        """Render a list of properties dictionaries"""
        properties = indent + ""
        for property in self.list_properties():
            if properties <> "":
                properties = properties + ",\n"
            properties = properties + indent + self.default_indent + \
                         property.render_self(self.default_indent)
        return properties


    def _render_properties_dictionary(self, indent=''):
        """Render the _properties dictionary"""
        if string.strip(self._render_properties()):
            dictionary = indent + self.default_indent + "_properties = (\n"
            dictionary = dictionary + self._render_properties(indent) + "\n"
            dictionary = dictionary + indent + (2 * self.default_indent) + ")\n"
        else:
            dictionary = indent + self.default_indent + "_properties = ()\n"
        return dictionary



class KlassRendrer(MethodKlassRendrer, PropertyKlassRendrer):
    """Adds methods for rendering"""

    def _render_license(self):
        """Render the license"""
        return self.license % {'author':self.author,
                               'author_email':self.author_email,
                               'authored_year':self.authored_year}


    def _render_name_and_subclasses(self):
        """Render the class name and any subclasses"""
        text = "class " + self.id
        bases = map(lambda x: x.split('/')[1], self.bases)
        text += "("+string.join(bases, ', ')+"):\n"
        return text


    def _render_bases_import_path(self):
        """Return the import path for the 'type'"""
        text = ''
        for base in self.bases:
            base = string.split(base, '/')
            text += 'from %s import %s\n' %(base[0], base[1])
        return text

           
    def _render_imports(self):
        """Return a string of needed imports"""
        text = self._render_bases_import_path() + "\n"
        text += "from Globals import DTMLFile, InitializeClass\n"
        text += "from AccessControl import ClassSecurityInfo\n"
        for subklass in self.list_subklasses():
            text += "import " + subklass.id + "\n"
        return text


    def _render_misc_definitions(self):
        """Render misc stuff"""
        text = self.default_indent + "meta_type = '" + self.meta_type + "'\n"
        return text + self.default_indent + "icon = '" + self.icon_path + "'\n"


    def _render_meta_types_and_accessor_calls(self):
        """Render the meta_types dictionary"""
        dict = "{'name': '%(id)s', 'action': '%(id)s_add_form'},"
        if self.list_subklasses():
            text = self.default_indent + "meta_types = (\n"
            for subklass in self.list_subklasses():
                text = text + (2 * self.default_indent) + \
                       dict % {'id': subklass.id} + "\n"
            text = text + (2 * self.default_indent) + ")\n\n"
        else:
            text = ""
        return text


    def _render_accessors(self):
        """Render the accessors for meta_types"""
        str_ = "%(meta_type)sAddForm = %(meta_type)s." + \
               "%(meta_type)sAddForm\n" + self.default_indent + \
               "manage_add%(meta_type)s = %(meta_type)s." + \
               "manage_add%(meta_type)s\n"
        text = ""
        for klass_ in self.list_subklasses():
            text += self.default_indent + str_ % {'meta_type': klass_.id}

        if text:
            text += "\n"
        return text


    def _render_misc_headers(self):
        """Render misc headers"""
        text = ""
        if self.misc_headers:
            text = "# $Id: Klass.py,v 1.1.2.1 2002/01/10 06:33:43 srichter Exp $\n"
            text += "# $Source: /cvs/ZOQLMethod/MakeProduct/Attic/Klass.py,v $\n"
            text += "__version__ = '$Revision: 1.1.2.1 $'[11:-2]\n\n"
            text += self.module_docstring + "\n"
        return text


    def _render_add_function(self):
        """Render the def *add() function and its DTMLFile"""
        text = "manage_add%(id)sForm = DTMLFile('dtml/%(id)s_add_form', globals())\n\n" % \
               {'id': self.id}
        text += """def manage_add%(id)s(self, id, REQUEST=None, **kw):
    \"\"\"Add an instance of the %(id)s class\"\"\"
    self._setObject(id, %(id)s(id, kw))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)\n""" % {'id': self.id}
        return text


    def _render_init_function(self):
        """Render the __init__(self, id, values={}) function"""
        text =  ('''    def __init__(self, id, values={}):\n'''
                 '''        """Initialize the class instance"""\n'''
                 '''        self.id = id\n'''
                 '''\n'''
                 '''        for property in self._properties:\n'''
                 '''            if property['id'] in values.keys():\n'''
                 '''                self._setPropValue(property['id'], values[property['id']])\n'''
                 '''            elif property.has_key('default'):\n'''
                 '''                self._setPropValue(property['id'], property['default'])\n''')
        return text


    def _render_initialize_class(self):
        """Render the InitializeClass call"""
        return "InitializeClass(%s)" % self.id + "\n"


    def render_self(self):
        """Render the klass instance in text.  This will be
        used when the product class will write itself out to
        the filesystem."""
        text = self._render_license() + "\n"
        misc_headers = self._render_misc_headers()
        if misc_headers:
            text += misc_headers + "\n"
        imports = self._render_imports()
        if imports:
            text += imports + "\n"
        text += self._render_add_function() + "\n"
        text += self._render_name_and_subclasses()
        text += self.default_indent + self.docstring + "\n\n"
        text += self._render_misc_definitions() + "\n"
        text += self._render_meta_types_and_accessor_calls()
        text += self._render_accessors()
        text += self._render_properties_dictionary() + "\n"
        text += self._render_set_security_info() + "\n"
        text += self._render_init_function() + "\n"
        text += self._render_methods(self.default_indent)
        text += self._render_initialize_class()
        return text


    def _render_for_class_structure(self, indent):
        """Return a line about ourself"""
        return indent + self.id + ' ' + self.meta_type + '\n' + \
               indent + '  ' + self.docstring + '\n'


    def render_class_structure(self, indent):
        """Return a string-representation of this klass and any
        subklasses.  The representation is nicely, and specifically,
        formatted for the render_class_structure() of the
        klass_product_rendrer class."""
        text = self._render_for_class_structure(indent)
        indent = indent + "  "
        for subklass in self.list_subklasses():
            text += subklass.render_class_structure(indent)
        return text


    def _render_set_security_info(self):
        """Return the ClassSecurityInfo()-string"""
        return self.default_indent + "security = ClassSecurityInfo()\n"



class Klass(MethodKlass, PropertyKlass, KlassRendrer):
    """A klass"""

    _properties = ({'id': 'module_docstring', 'type': 'string', 'default': "'''doc string'''"},
                   {'id': 'register',         'type': 'int',    'default': 0},
                   ) + BasicKlass._properties
    

    def _setup_default(self):
        """Add default methods and properties"""
        body = self.default_indent + "self.id = id\n\n"
        body = body + self.default_indent + "for property in self._properties:\n"
        body = body + (2 * self.default_indent) + \
               "if property['id'] in values.keys():\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], values[property['id']])\n"
        body = body + (2 * self.default_indent) + \
            "elif property.has_key('default'):\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], property['default'])\n"

        self.add_method(
            id='__init__', arguments=['self', 'id', ['values', {}]], body=body,
            docstring='"""Initialize an instance of the class"""')


    def _write_subklasses(self, directory):
        """Write all subclasses"""
        for subklass in self.list_subklasses():
            subklass.write_self(directory)


    def _write_add_form(self, directory):
        """Write the add_form.dtml-file"""
        file = open(os.path.join(
            directory, 'dtml', self.id + '_add_form.dtml'), 'w')
        file.write(open(os.path.join(ROOT_DIR,
            'templates', 'add_form.dtml')).read() % {'klass': self.id})
        file.close()


    def _write_init(self, directory):
        """Write the appropriate lines in the __init__.py"""
        text  = '    import ' + self.id + '\n'
        text += '    context.registerClass(%s.%s,\n' %(self.id, self.id)
        text += '        constructors=(%s.manage_add%sForm,\n' %(self.id, self.id)
        text += '                      %s.manage_add%s)\n' %(self.id, self.id)
        text += '        )\n'

        file = open(os.path.join(directory, '__init__.py'), 'a')
        file.write(text)
        file.close()

                        
    def write_self(self, directory):
        """Write the rendered klass to a file"""
        file = open(os.path.join(directory, self.id) + '.py', 'w')
        file.write(self.render_self())
        file.close()
        self._write_init(directory)
        self._write_add_form(directory)
        self._write_subklasses(directory)
