"""A collection of classes for creating and maintaining methods within
klass instances.  As you can see, the method class does not know how
to render itself, it needs to subclass the method_rendrer to be able to
do that."""

import types, string


class BasicMethod:
    """A basic method"""

    _properties = ({'id': 'id',             'type': 'string', 'default': 'id'},
                   {'id': 'arguments',      'type': 'list',   'default': ['self']},
                   {'id': 'docstring',      'type': 'string', 'default': "'''doc string'''"},
                   {'id': 'default_indent', 'type': 'string', 'default': '    '},
                   {'id': 'body',           'type': 'string', 'default': '    pass'},
                   {'id': 'security',       'type': 'string', 'default': 'public'},
                   {'id': 'permission',     'type': 'string', 'default': ''})


    def __init__(self, **kw):
        """Initialize an instance of the class"""

        for property in self._properties:
            if property['id'] in kw.keys():
                setattr(self, property['id'], kw[property['id']])
            else:
                if property['type'] == 'list':
                    default = list(property['default'])
                else:
                    default = property['default']
                setattr(self, property['id'], property['default'])

        if 'self' not in kw['arguments']:
                arguments = ['self'] + kw['arguments']


    def get_arguments(self):
        """Return the arguments for the method"""
        return self.arguments



class MethodRendrer(BasicMethod):
    """Provides methods for rendering a basic_method instance"""

    def render_security_declaration(self, indent):
        """Return the security declaration"""
        if self.security == 'public':
            return indent + "security.declarePublic('%s')\n" % self.id
        elif self.security == 'private':
            return indent + "security.declarePrivate('%s', '%s')\n" % \
                   (self.permission, self.id)


    def render_arguments(self):
        """Return the arguments-string"""
        arguments = ""
        for argument in self.get_arguments():
            if arguments <> "":
                arguments = arguments + ", "
            if type(argument) == types.ListType:
                arguments = arguments + argument[0] + "=" +\
                            str(argument[1])
            else:
                arguments = arguments + argument
        return arguments


    def render_body(self, indent=''):
        """Return the body of the method"""
        if self.body:
            text = ""
            for line in string.split(self.body, '\n'):
                text = text + indent + line + "\n"
            return text
        else:
            return indent + self.default_indent + "pass\n"


    def render_self(self, indent=''):
        """Render the method instance in text.  This will be
        used when the product class will write itself out to
        the filesystem."""
        text = self.render_security_declaration(indent)
        text += indent + "def " + self.id + "(" + \
               self.render_arguments() + "):\n"
        text += indent + self.default_indent + self.docstring + "\n"
        text += self.render_body(indent)
        return text



class Method(MethodRendrer):
    """This class represents a Python method."""
    pass




