"""A bunch of classes that mimick some Product behaviour."""
import utils, string, os

try:
    from Globals import package_home
    ROOT_DIR = package_home(globals())
except:
    ROOT_DIR = ''

class BasicProduct:
    """A basic product"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.author = ''
        self.author_email = ''
        self.authored_year = ''



class KlassProduct(BasicProduct):
    """This class provides methods for manipulating classes"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.klasses = []


    def add_klass(self, klass):
        """Add a klass instance"""
        if not utils.find_entry(self.list_klasses(), klass.id):
            self.klasses.append(klass)
        else:
            raise "DuplicateKlass", "There already exists a " + \
                  "klass with that 'id'"
        return klass


    def list_klasses(self):
        """Return a list of all klass instances"""
        return self.klasses


    def remove_klass(self, klass_id):
        """Remove the klass with the id 'klass_id'"""
        instance = utils.find_entry(self.list_klasses(), klass_id)
        if instance: self.klasses.remove(instance)


    def get_klass(self, klass_id):
        """Return the klass instance with id 'klass_id'"""
        return utils.find_entry(self.list_klasses(), klass_id)



class KlassProductRendrer(KlassProduct):
    """This class provides methods for rendering the klass layout in
    klass_products"""

    def render_class_structure(self):
        """Return a nice string-representation of the class
        structure"""
        text = self.id + "\n"
        text += "  " + self.docstring + "\n"
        indent = "  "
        for klass in self.list_klasses():
            text += klass.render_class_structure(indent)
        return text



class ZopeProduct(KlassProduct, KlassProductRendrer):
    """A class that provides methods for setting up a Zope Python
    Product"""

    _properties = ({'id': 'id',             'type': 'string', 'default': 'id'},
                   {'id': 'license',        'type': 'string', 'default': ''},
                   {'id': 'docstring',      'type': 'string', 'default': "'''doc string'''"},
                   {'id': 'default_indent', 'type': 'string', 'default': '    '},
                   {'id': 'klasses',        'type': 'list',   'default': []},
                   {'id': 'directory',      'type': 'string', 'default': ''},
                   {'id': 'author',         'type': 'string', 'default': 'Foo Bar'},
                   {'id': 'author_email',   'type': 'string', 'default': 'foo@bar.com'},
                   {'id': 'authored_year',  'type': 'string', 'default': ''},
                   {'id': 'form_creator',   'type': 'string', 'default': 'self'})
                   

    def __init__(self, **kw):
        """Initialize an instance of the class"""

        for property in self._properties:
            if property['id'] in kw.keys():
                setattr(self, property['id'], kw[property['id']])
            else:
                if property['type'] == 'list':
                    default = list(property['default'])
                else:
                    default = property['default']
                setattr(self, property['id'], property['default'])
                
        self.directory = os.path.join(self.directory, self.id)

        
    def _add___init__py(self):
        """Add __init__.py file to the product directory"""
        klasses_to_register = []
        if self.list_klasses():
            for klass in self.list_klasses():
                if klass.register:
                    klasses_to_register.append(klass)
            if klasses_to_register:
                module_list = string.join(map(lambda x: getattr(x,'id'),klasses_to_register), ', ')
            else:
                klasses_to_register = [self.list_klasses()[0]]
                module_list = klasses_to_register[0].id
        else:            
            module_list = 'None'
        file = open(os.path.join(ROOT_DIR, 'templates', '__init__header.py'))
        header = file.read() % {'module_list': module_list, 'license': self.license, 
                                'id':self.id, 'prod_id':self.id}
        file.close()
        file = open(os.path.join(self.directory, '__init__.py'), 'w')                        
        file.write(header)
        for klass in klasses_to_register:
            file.write(open(os.path.join(ROOT_DIR, 'templates', '__init__footer.py')).read() % {'id': klass.id})
        file.close()


    def _add_versiontxt(self):
        """Add version.txt file to the product directory"""
        file = open(os.path.join(self.directory, 'version.txt'), 'w')
        file.write('0.0.1')
        file.close()


    def _add_help_directory(self):
        """Add help/ directory"""
        os.mkdir(os.path.join(self.directory, 'help'))


    def _add_www_directory(self):
        """Add the www/ directory"""
        os.mkdir(os.path.join(self.directory, 'www'))


    def _add_dtml_directory(self):
        """Add the dtml/ directory"""
        os.mkdir(os.path.join(self.directory, 'dtml'))


    def _create_directory(self):
        """Create the directory in Products/"""
        if os.path.isdir(self.directory):
            raise "DirectoryExists", "The product directory " + \
                  self.directory + " already exist.  Please remove first."
        else:
            os.mkdir(self.directory)


    def _write_refresh_txt(self):
        file = open(os.path.join(self.directory, 'refresh.txt'), 'w')
        file.write('File that activates Product Refresh functionality.')
        file.close()        


    def _setup_defaults(self):
        """Set up some defaults"""
        self._create_directory()
        self._add___init__py()
        self._add_versiontxt()
        self._add_help_directory()
        self._add_www_directory()
        self._add_dtml_directory()
        self._write_refresh_txt()


    def _write_klasses(self):
        """Write all klasses to files in self.directory"""
        for klass in self.list_klasses():
            klass.write_self(self.directory)


    def write_self(self):
        """Write the product to the filesystem"""
        self._setup_defaults()
        self._write_klasses()


    def delete_self(self):
        """Delete the product from the filesystem"""
        if not os.path.isdir(self.directory):
            raise "DirectoryDoesNotExist", "The product directory " + \
                  self.directory + " does not exist."
        else:
            os.system('rm -r ' + self.directory)








