# This code is copyrighted by Stephan Richter and
# Thomas Foerster in September, 2001.
#
# written by Stephan Richter
# Last updated: October 4, 2001
import string


class Operator:

    is_operator = 1

    reprs = []

    def __init__(self, lhs, rhs):
        ''' '''
        if type(lhs) is type(()):
            self.lhs = lhs[1]
        else:
            self.lhs = lhs

        if type(rhs) == type(()):
            self.rhs = rhs[1]
        else:
            self.rhs = rhs


    def __getitem__(self, index):
        ''' '''
        if index == 0:
            return self.lhs
        elif index == 1:
            return self.rhs
        else:
            return None
        

    def __str__(self):
        ''' '''
        return str(self.lhs) + ' ' + self.reprs[0] + ' ' + str(self.rhs)


    def _get_values(self, object):
        ''' '''
        result = []

        for side_str in ('lhs', 'rhs'):
            side = getattr(self, side_str)
            
            if hasattr(side, 'get_reference'):
                try:
                    side = getattr(object, side.get_reference())
                except AttributeError: pass

            
            if callable(side):
                try:
                    side = side(object)
                except:
                    side = side()
            
            result.append(side)

        return tuple(result)


    def execute(self, object):
        ''' '''
        pass


    def __call__(self, object):
        return self.execute(object)



# ** Start of Boolean Operators Implementation **

class BooleanOperator(Operator):
    pass


class Equal(BooleanOperator):

    reprs = ['==', 'equal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        return lhs == rhs


class NotEqual(BooleanOperator):

    reprs = ['<>', '!=', 'nequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs != rhs


class GreaterThanEqual(BooleanOperator):

    reprs = ['>=', 'gtequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs >= rhs


class LessThanEqual(BooleanOperator):

    reprs = ['<=', 'ltequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs <= rhs


class GreaterThan(BooleanOperator):

    reprs = ['>', 'greater']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs > rhs


class LessThanEqual(BooleanOperator):

    reprs = ['<', 'lesser']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs < rhs


class And(BooleanOperator):

    reprs = ['&', 'and']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs and rhs


class Or(BooleanOperator):

    reprs = ['|', 'or']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs or rhs


class Xor(BooleanOperator):

    reprs = ['^', 'xor']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return (lhs or rhs) and not (lhs and rhs)


class Not(BooleanOperator):

    reprs = ['!', 'not']

    def execute(self, object):
        rhs = self._get_values(object)[1]        
        return not rhs


class In(BooleanOperator):

    reprs = ['in']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs in rhs


class Is(BooleanOperator):

    reprs = ['is']

    def execute(self, object):
        lhs, rhs = self._get_values(object)        
        return lhs is rhs


class DeclaritiveOperator(Operator):
    ''' '''

    def _store(self, object, content):
        lhs = self.lhs.get_reference()
        if len(string.split(lhs, '.')) == 1:
            if lhs == 'data':
                object.manage_edit(content, object.title)
            elif lhs == 'id':
                object.aq_parent.manage_renameObject(object.getId(), content)
            else:
                setattr(object, lhs, content)
        else:
            # We deal with a ZClass
            propertysheet_id, property_id = string.split(self.lhs.get_reference(), '.')
            propertysheet = getattr(object.propertysheets, propertysheet_id)
            propertysheet._updateProperty(property_id, content)
    


class SetEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['=']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        self._store(object, rhs)
            

class PlusEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['+=', 'plusequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        lhs += rhs
        self._store(object, lhs)


class MinusEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['-=', 'minusequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        lhs -= rhs
        self._store(object, lhs)


class MulEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['*=', 'mulequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        lhs *= rhs
        self._store(object, lhs)


class DivEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['/=', 'divequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        lhs /= rhs
        self._store(object, lhs)


class ModEqual(DeclaritiveOperator):
    ''' '''

    reprs = ['%=', 'modequal']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        lhs %= rhs
        self._store(object, lhs)



class ArithmeticOperator(Operator):
    pass


class Add(ArithmeticOperator):

    reprs = ['+', 'plus']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs + rhs
    

class Subtract(ArithmeticOperator):

    reprs = ['-', 'minus']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs - rhs


class Multiply(ArithmeticOperator):

    reprs = ['*', 'times']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs ** rhs


class Power(ArithmeticOperator):

    reprs = ['**', '^', 'to the']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs ** rhs


class Divide(ArithmeticOperator):

    reprs = ['/', 'divide']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs / rhs


class Modular(ArithmeticOperator):

    reprs = ['%', 'mod']

    def execute(self, object):
        lhs, rhs = self._get_values(object)
        if lhs is None: return rhs
        if rhs is None: return lhs
        return lhs % rhs

