from Globals import package_home
from App     import Product, RefreshFuncs
from OFS     import Application
from Products.ZOQLMethod.MakeProduct import Builder
import Products

import string, os
import Operators, CatalogOperators
import ZOQLExceptions

### Helper functions
def get_identifier_name(elem):
    return getattr(elem, 'attr', elem)

def unwrap(object):
    if hasattr(object, 'aq_base'):
        return object.aq_base
    return object

class Reference:
    '''A class that represents an object reference'''

    def __init__(self, ref_string):
	self.ref_list = string.split(ref_string, '.')
        
    def __str__(self):
        return self.get_path()

    def get_path(self):
        return string.join(self.ref_list, '/')

    def get_reference(self):
        return string.join(self.ref_list, '.')

    def get_object(self, base_object, default=None):
        # defun???
        return base_object.unrestrictedTraverse(self.ref_list, default)
        
    
class PropertyRename:
    '''A class that handles: ref_to_prop AS prop_name'''
    
    def __init__(self, prop_ref, new_name):
        self.prop_ref = prop_ref
        self.new_name = new_name.get_reference()

    def __str__(self):
        return str(self.prop_ref) + ' AS ' + self.new_name

    def get_new_name(self):
        return self.new_name

    def get_prop_ref(self):
        return self.get_prop_ref


class ReturnType:
    '''A class that defines the return object type: BRAIN, OBJECT or CLASS'''
    
    def __init__(self, type='BRAIN', klass=None):

        self.type = string.lower(type)
        self.klass = klass


class ClassCreationOptions:
    """A class that collects options for (Z)Class creation"""

    def __init__(self):
        self._options = {'create_factory': 0, 'zope_object': 0}

    def addOption(self, option):
        """Add an option, 'option' is a SetEqual object --> see Operators.py"""
        if hasattr(option[1], 'execute'):
            self._options[option[0].get_reference()] = option[1].execute()
        else:
            self._options[option[0].get_reference()] = option[1]

    def __getitem__(self, option):
        return self._options[option]


class Compare:
    """Needed for orderby in selections"""
    def __init__(self, orderby):
        self.orderby = orderby

    def __call__(self, x, y):

        for prop in self.orderby:
            if prop['property'] != 'id':
                result = cmp(getattr(x, prop['property'], None), getattr(y, prop['property'], None))                
            else:
                result = cmp(x.getId(), y.getId())                

            if string.lower(prop['direction']) == 'desc':
                result = -result
            if result != 0: break

        return result



class Property:
    def __init__(self, name, type, default):
        self._name = name
        self._type = type
        self._default = default
        
    def name(self):
        return self._name

    def type(self):
        return self._type

    def default(self):
        return self._default
    

class PropertySheet:
    def __init__(self, name, properties):
        self._name = name
        self._properties = properties
        
    def name(self):
        return self._name

    def properties(self):
        return self._properties

## Creational Helper Classes
    
class ZClassProductCreation:
    """
    A class that encapsulates the creation of a zclass product
    """

    def __init__(self, name):
        self.name = name

    def __call__(self, base_object):
        base_object.getPhysicalRoot().Control_Panel.Products.manage_addProduct(self.name, self.name+' product')

        return hasattr(base_object.getPhysicalRoot().Control_Panel.Products,
                       self.name)



class PythonProductCreation:
    """
    A class that encapsulates the creation of a python product
    """
    def __init__(self, name):
        self.name = name

    def __call__(self, base_object):
        product = Builder.zope_product_builder(id = self.name,
                                               directory = os.path.join(package_home(globals()), '../..'),
                                               author = 'ZOQL Method',
                                               author_email = 'srichter@cbu.edu',
                                               authored_year = '2002')
        product.write_self()
        meta_types = []
        Application.install_product(base_object, Products.__path__[0], self.name, meta_types,
                                    Application.get_folder_permissions())

        Products.meta_types = Products.meta_types+tuple(meta_types)
        


class PythonClassCreation:
    """
    Encapsulate creation of a specific python class
    """
    def __init__(self, prod_id, class_id, attributes=[], options=None,
                 propertysheets=[], bases=[]):

        self.class_id = class_id
        self.prod_id = prod_id
        self.bases = bases
        self.attributes = attributes
        self.propertysheets = propertysheets
        self.options = options

        self.meta_type = self.class_id

        for attribute in self.attributes:
             if attribute.lhs.get_reference() == 'meta_type':
                 self.meta_type = attribute.rhs


    def __call__(self, base_object):
        klass = Builder.klass_builder(id = self.class_id,
                                      bases = self.bases,
                                      meta_type = self.meta_type,
                                      author = 'ZOQL Method',
                                      author_email = 'srichter@cbu.edu',
                                      authored_year = '2002')

        for sheet in self.propertysheets:
            for property in sheet.properties():
                klass.add_property(Builder.property_builder(
                                   id = property.name(),
                                   type = property.type(),
                                   default = property.default()))

        klass.write_self(os.path.join(package_home(globals()), '../..', self.prod_id))
        RefreshFuncs.performFullRefresh(base_object._p_jar, self.prod_id)


class ZClassCreation:
    """
    Encapsulate creation of a specific zclass
    """

    def __init__(self, prod_id, class_id, attributes=[], options=None,
                 propertysheets=[], bases=[]):
        self.class_id = class_id
        self.prod_id = prod_id
        self.bases = bases
        self.attributes = attributes
        self.propertysheets = propertysheets
        
        if not options: options = ClassCreationOptions()
        self.options = options

        self.meta_type = self.class_id

        for attribute in self.attributes:
             if attribute.lhs.get_reference() == 'meta_type':
                 self.meta_type = attribute.rhs


    def __call__(self, base_object):
        product = base_object.getPhysicalRoot().Control_Panel.Products[self.prod_id]
        
        product.manage_addZClass(self.class_id, '', self.bases,
                                 self.meta_type,
                                 self.options['create_factory'],
                                 None,
                                 self.options['zope_object'])

        class_instance = product[self.class_id]

        #XXX ??? set properties ???
        ## for attr in attributes:
        ##    attr.execute(class_instance)

        for sheet in self.propertysheets:
            PropertySheetCreation(sheet)(class_instance)



class PropertySheetCreation:
    """
    Create a specific property sheet for base objects
    """
    def __init__(self, sheet=[]):
        self.sheet = sheet

    def __call__(self, base_object):
        id = self.sheet.name()
        title = id + " Property Sheet" 
        
        base_object.propertysheets.common.manage_addCommonSheet(id, title)

        property_sheet = base_object.propertysheets.common[id]
        for property in self.sheet.properties():
            PropertyCreation(property)(property_sheet)



class PropertyCreation:
    """
    Create a specific property for base objects
    """
    def __init__(self, property):
        self.property = property

    def __call__(self, base_object):
        id = self.property.name()
        prop_type = self.property.type()
        default = self.property.default()
        base_object.manage_addProperty(id, default, prop_type)



class InstanceCreation:
    """
    Create a class instance in a container
    """
    def __init__(self, meta_type, object_id, container_path=None, assignments=[]):
        self.meta_type = meta_type
        self.object_id = object_id
        self.container_path = container_path
        self.assignments = assignments

    def __call__(self, base_object):
        container = base_object.unrestrictedTraverse(self.container_path)
        
        klass = None
        for entry in container.all_meta_types():
            if entry['name'] == self.meta_type:
                if entry.has_key('instance'):
                    klass = entry['instance']
                else:
                    for zclass in base_object.getPhysicalRoot().Control_Panel.Products[entry['product']].objectValues('Z Class'):
                        if zclass._zclass_.meta_type == self.meta_type:
                            klass = zclass
                            break
                if not klass:
                    raise 'NotFoundError', "Meta Type '%s' not found" % self.meta_type
        if not klass:
            raise 'NotAllowed', "Meta Type '%s' not allowed for %s" % (self.meta_type, container.getId())
        
        instance = klass(self.object_id) #XXX what about other constructor params ???
        instance._setId(self.object_id)
        container._setObject(self.object_id, instance)

        object = container[self.object_id]

        #XXX a bunch of quick hacks Unittests, Unittests, Unittests...
        for assignment in self.assignments:
            if assignment.lhs.get_reference() != 'id':
                assignment.execute(object)



class IndexCreation:
    def __init__(self, name, selected_meta_types=None,
                 index_list=[], conditions=[],
                 recursive=0):
        self.name = name
        self.meta_types = selected_meta_types
        self.index_list = index_list
        self.conditions = conditions
        self.recursive = recursive


    def emptyCatalog(self, base_object):
        base_object.manage_addProduct['ZCatalog'].manage_addZCatalog(self.name, 'ZOQL Index')

        catalog = base_object[self.name]

        for ic in ('title', 'bobobase_modification_time'):
            catalog.delColumn(ic)
            catalog.delIndex(ic)

        catalog.delColumn('summary')
        catalog.delIndex('PrincipiaSearchSource')

        return catalog

    
    def __call__(self, base_object):
        catalog = self.emptyCatalog(base_object)

        for new_index in self.index_list:            
            if new_index['name'] not in catalog.indexes():
                index_type = string.capitalize(new_index['type']) + 'Index'
                catalog.manage_addIndex(new_index['name'], index_type)

        select = Selection(selected_meta_types = self.meta_types,
                           conditions = self.conditions,
                           recursive = self.recursive,
                           use_indexes = 0)
        objects = select(base_object)
        
        for object in objects:
            catalog.catalog_object(object)


class SequenceCreation:
    """
    A Class that creates a new sequence
    """

    def __init__(self, name='sequence', increment=1,
                 minvalue=0, maxvalue=1000, start=0):
        """ """
        self.name = name
        self.increment = increment or 1
        self.minvalue = minvalue or 0
        self.maxvalue = maxvalue or 1000000
        self.start = start or 0


    def __call__(self, base_object):
        """ """

        from Sequence import Sequence
        base_object._setObject(self.name.get_reference(),
                               Sequence(self.name.get_reference(),
                                        self.increment, self.minvalue,
                                        self.maxvalue, self.start))

        

class AddAlteration:
    """
    A class that adds new properties and methods to an existing class.
    """
    
    def __init__(self, class_ref, object_type, object_ref):

        self.class_ref = class_ref
        self.object_type = object_type
        self.object_ref = object_ref


    def __call__(self, base_object):
        """ """

        if string.lower(self.object_type) == 'method':
            raise NotImplemented

        name = self.object_ref.name()

        if len(string.split(name, '.')) == 2:
            class_object = base_object.unrestrictedTraverse('/Control_Panel/Products/'+
                                                            self.class_ref.get_path())

            sheet, self.object_ref._name = string.split(name, '.')
            PropertyCreation(self.object_ref)(class_object.propertysheets.common[sheet])

        elif len(string.split(name, '.')) == 1:
            class_object = base_object.unrestrictedTraverse(self.class_ref.get_path())

            PropertyCreation(self.object_ref)(class_object)



class RenameAlteration:
    """
    A class that renames properties and methods to an existing class.
    """
    
    def __init__(self, class_ref, old_name, new_name):

        self.class_ref = class_ref
        self.old_name = old_name
        self.new_name = new_name


    def __call__(self, base_object):
        """ """

        old_name = self.old_name.get_reference()

        if len(string.split(old_name, '.')) == 2:
            class_object = base_object.unrestrictedTraverse('/Control_Panel/Products/'+
                                                            self.class_ref.get_path())
            sheet_id, old_id = string.split(old_name, '.')
            sheet = class_object.propertysheets.common[sheet_id]
            
        if len(string.split(old_name, '.')) == 1:
            class_object = base_object.unrestrictedTraverse(class_ref.get_path())
            old_id = old_name
            sheet = class_object

        prop_type = sheet.getPropertyType(old_id)
        prop_value = sheet.getProperty(old_id)

        # delete ...
        sheet._delProperty(old_id)

        # Add the property again ...
        new_name = self.new_name.get_reference()
        new_name = string.split(new_name, '.')[-1]

        sheet.manage_addProperty(new_name, prop_value, prop_type)



class DeleteAlteration:
    """
    A class that deletes properties and methods to an existing class.
    """
    
    def __init__(self, class_ref, property_ids):

        self.class_ref = class_ref
        self.property_ids = property_ids


    def __call__(self, base_object):
        """ """

        try:
            class_object = base_object.unrestrictedTraverse('/Control_Panel/Products/'+
                                                            self.class_ref.get_path())
            
            for id in self.property_ids:
                id = id.get_reference()
                sheet, prop = string.split(id, '.')
                class_object.propertysheets.common[sheet]._delProperty(prop)

        except:
            class_object = base_object.unrestrictedTraverse(self.class_ref.get_path())

            for id in self.property_ids:
                class_object._delProperty(id.get_reference())



class InstanceUpdate:
    """
    A class that can update an object (or a class instance).
    """

    def __init__(self, selected_meta_types=None, searched_containers=[],
                  assignments=[], conditions=None, recursive=1):
        """Initialize"""
        
        # first complete the constraints
        if selected_meta_types and 'ALL' not in map(lambda x: x.get_reference(), selected_meta_types):
            self.conditions = self.restrict_meta_types(selected_meta_types, conditions)
        else:
            self.conditions = conditions

        self.searched_containers = searched_containers
        self.assignments = assignments
        self.recursive = recursive


    def restrict_meta_types(self, meta_types, conditions):
        """
        Add Conditions to the tree to constrain to the given set of
        meta types
        """
        
        def _build_meta_type_condition(tree, item):
            c = Operators.Equal(Reference('meta_type'), item)
            if not tree:
                return c
            return Operators.Or(tree, c)
        
        meta_types = map(lambda x: x.get_reference(), meta_types)
        meta_type_cond = reduce(_build_meta_type_condition, meta_types, None)

        if conditions is not None:
            conditions = Operators.And(conditions, meta_type_cond)
        else:
            conditions = meta_type_cond

        return conditions


    def collect_candidates(self, base_object):
        """Search the object tree for possible result objects"""

        objects = []
        
        search_base = unwrap(base_object)
        if hasattr(search_base, 'objectValues'): 

            if self.searched_containers:
                for container_path in self.searched_containers:
                    container = container_path.get_object(base_object)
                    if hasattr(container, 'objectValues'):
                        objects += list(container.objectValues())
            else:
                objects = base_object.objectValues()

        return objects


    def __call__(self, base_object):
        """ """
        if base_object:        
            for object in self.collect_candidates(base_object):
                dflag = 0
                if (hasattr(object, '_p_changed') and
                    (object._p_changed == None)):
                    dflag = 1

                if (not self.conditions) or (self.conditions.execute(object)):
                    for assignment in self.assignments:
                        assignment.execute(object)
                    dflag = 0

                if self.recursive:
                    # no need to restrict meta types, since 'compiled' in
                    # conditions already
                    InstanceUpdate(return_type=self.return_type,
                                   recursive = self.recursive,
                                   use_indexes = self.use_indexes,
                                   conditions = self.conditions)(object)
                        
                if dflag:
                    object._p_deactivate()


            
class Selection:
    """
    A Class that stores selection related data and performs this
    selection for a given base object when called
    """

    def __init__(self, selected_meta_types=None, return_type=None,
                 searched_containers=[],
                 conditions=None, orderby=[],
                 limit={}, recursive=0, use_indexes=1):

        # first complete the constraints
        if selected_meta_types and 'ALL' not in map(lambda x: x.get_reference(), selected_meta_types):
            self.conditions = self.restrict_meta_types(selected_meta_types, conditions)
        else:
            self.conditions = conditions

        self.return_type = return_type
        self.searched_containers = searched_containers
        self.limit = limit
        self.orderby = orderby
        self.use_indexes = use_indexes
        self.recursive = recursive


    def restrict_meta_types(self, meta_types, conditions):
        """
        Add Conditions to the tree to constrain to the given set of
        meta types
        """
        
        def _build_meta_type_condition(tree, item):
            c = Operators.Equal(Reference('meta_type'), item)
            if not tree:
                return c
            return Operators.Or(tree, c)
        
        meta_types = map(lambda x: x.get_reference(), meta_types)
        meta_type_cond = reduce(_build_meta_type_condition, meta_types, None)

        if conditions is not None:
            conditions = Operators.And(conditions, meta_type_cond)
        else:
            conditions = meta_type_cond

        return conditions


    def find_indexes(self, base_object):

        indexes = []
        for catalog in base_object.objectValues('ZCatalog'):
            if catalog.title == 'ZOQL Index':
                indexes.append(catalog)

        return indexes


    def select_from_indexes(self, base_object):
        """Do a select with catalog indices"""

        result = []
        indexes = self.find_indexes(base_object)
        if indexes:
                c = CatalogOperators.convert_operator(self.conditions)

        # for every index that was found, do the search
        for index in indexes:
            # Interesting, sometimes we do not have any conditions!
            if c:
                result_brains = c.execute(index)
            else:
                result_brains = index()

            if self.return_type and self.return_type.type == 'object':
                # Convert the resulting brains to actual objects; yeah!
                for brain in result_brains:
                    object = index.getobject(brain.data_record_id_)
                    result.append(object)
            else:
                result += list(result_brains)

        return result


    def collect_candidates(self, base_object):
        """Search the object tree for possible result objects"""

        objects = []
        
        search_base = unwrap(base_object)
        if hasattr(search_base, 'objectValues'): 

            if self.searched_containers:
                for container_path in self.searched_containers:
                    container = container_path.get_object(base_object)
                    if hasattr(container, 'objectValues'):
                        objects += list(container.objectValues())
            else:
                objects = base_object.objectValues()

        return objects


    def __call__(self, base_object):
        result = []

        if self.use_indexes:
            result = self.select_from_indexes(base_object)

        # Okay, we have no more index, so we simply walk the tree.
        if base_object:        
            for object in self.collect_candidates(base_object):
                # Has anyone a clue, why this is done? And what does dflag stand for?
                dflag = 0
                if (hasattr(object, '_p_changed') and
                    (object._p_changed == None)):
                    dflag = 1

                if (not self.conditions) or (self.conditions.execute(object)):
                    result.append(object)
                    dflag = 0

                # Working recursively? Let's look inside the object as well
                if self.recursive:
                    # no need to restrict meta types, since 'compiled' in
                    # conditions already
                    subselection = Selection(return_type=self.return_type,
                                             recursive = self.recursive,
                                             use_indexes = self.use_indexes,
                                             conditions = self.conditions)
                    result += subselection(object)
                        
                # Do you guys know what that means?
                if dflag:
                    object._p_deactivate()

        # Let's order the result, based on the information passed
        if self.orderby:
            result.sort(Compare(self.orderby))

        # Mmmh, it would be so much more efficient to do the limit
        # before sorting,
        # but we can't do that of course...
        if self.limit:
            end = self.limit['start'] + self.limit['size']
            if end >= len(result):
                end = len(result)
            result = result[self.limit['start']:end]

        return result


    
class Deletion:
    """
    A Class that stores deletion related data and performs this
    selection for a given base object when called
    """

    def __init__(self, selected_meta_types=None,
                 searched_containers=[],
                 conditions=None, 
                 recursive=0, use_indexes=1):

        # first complete the constraints
        if selected_meta_types:
            self.conditions = self.restrict_meta_types(selected_meta_types, conditions)
        else:
            self.conditions = conditions

        self.searched_containers = searched_containers
        self.use_indexes = use_indexes
        self.recursive = recursive


    def restrict_meta_types(self, meta_types, conditions):
        """
        Add Conditions to the tree to constrain to the given set of
        meta types
        """
        def _build_meta_type_condition(tree, item):
            c = Operators.Equal(Reference('meta_type'), item)
            if not tree: return c
            return Operators.Or(tree, c)
        
        meta_types = map(lambda x: x.get_reference(), meta_types)
        meta_type_cond = reduce(_build_meta_type_condition, meta_types, None)
            
        if conditions is not None:
            conditions = Operators.And(conditions, meta_type_cond)
        else:
            conditions = meta_type_cond

        return conditions


    def collect_candidates(self, base_object):
        """
        Search the object tree for possible objects to delete
        Returns list with (object, container) tuples
        """

        objects = []
        
        search_base = unwrap(base_object)
        if hasattr(search_base, 'objectValues'): 

            if self.searched_containers:
                for container_path in self.searched_containers:
                    container = container_path.get_object(base_object)
                    if hasattr(container, 'objectValues'):
                        result = map(lambda x,y=container: (x,y),
                                     container.objectValues())
                        objects = objects + result
            else:
                objects = map(lambda x,y=base_object: (x,y),
                                     base_object.objectValues())

        return objects


    def __call__(self, base_object):
        if base_object:
            for object,container in self.collect_candidates(base_object):
                if (not self.conditions) or (self.conditions.execute(object)):
                    container.manage_delObjects(object.getId())
                    
                    if ( object.meta_type == 'Product' and
                         filter(lambda x, p=Products: hasattr(getattr(p, x), 'initialize'), dir(Products))):
                        # now delete the product from directory 
                        prod = Builder.zope_product_builder(id=object.id,
                                                     directory=os.path.join(package_home(globals()), '..', '..'))
                        prod.delete_self()

                elif self.recursive:
                    # as usual meta type conditions compiled in
                    s = Deletion(conditions = self.conditions,
                                 recursive = self.recursive)
                    s(object) #perform recursive descent
