# This code is copyrighted by Thomas Foerster and Stephan Richter
# in September, 2001.
#
# written by Thomas Foerster and Stephan Richter
# Last updated: October 4, 2001

# ZOQL grammar => see help/Syntax.dtml for description

#TODO:
# * extend names to allow all Zopish weirdness --> This can be done by using " " or ' '.

import string

#############################################################################
# ZOQL's basic elements are statements. These might be concatenated to
# ZOQL Scripts for convenience.
#############################################################################

script = """
oql_script ::
@R oql_script1 :: oql_script >> statement_list

@R statement_list1 :: statement_list >> statement
@R statement_list2 :: statement_list >> statement statement_list
"""

### several basic statements are defined

#############################################################################
# CREATE PRODUCT defines a new product within Zope. Depending on the given
# product type specifier it will be a ZClass or a full fledged Python product
#
# Syntax(1):
# CREATE PRODUCT name AS type specifier
#
# Alternative Form(9):
# CREATE type specifier PRODUCT name
#
# The type specifier may be either ZCLASS or PYTHON
#############################################################################

#XXX what about making product_type optional and providing ZCLASS as default
#XXX Decide which form fits better 1 or 9
create_prod_statement = """
@R statement_create1 :: statement >>
    CREATE PRODUCT identifier AS product_type ;

## The product type can have only two values: ZCLASS and PYTHON -- Tested
@R product_type1 :: product_type >> ZCLASS
@R product_type2 :: product_type >> PYTHON
"""

#############################################################################
# CREATE CLASS defines a new class
#############################################################################

create_class_statement = """
@R statement_create2 :: statement >>
    CREATE CLASS identifier IN identifier 
    opt_assignment opt_option opt_propertysheet_declaration opt_inherit ;

## Describes what object types to inherit in the new class -- Tested
@R opt_inherit0 :: opt_inherit >>
@R opt_inherit1 :: opt_inherit >> INHERIT reference_list

## There are some special options for the constructor of a class;
## so define them here  -- Tested
@R opt_option0 :: opt_option >>
@R opt_option1 :: opt_option >> OPTION assignment_list

@R opt_propertysheet_declaration0 :: opt_propertysheet_declaration >>
@R opt_propertysheet_declaration1 :: opt_propertysheet_declaration >>
    DECLARE propertysheet_list
"""

##########################################################################
# Propertysheet declarations
##########################################################################

propertysheets = """
## PropertySheets -- Tested
@R propertysheet_list1 :: propertysheet_list >> propertysheet
@R propertysheet_list2 :: propertysheet_list >> propertysheet , propertysheet_list

@R propertysheet1 :: propertysheet >> identifier { opt_prop_declaration }
@R propertysheet2 :: propertysheet >> string_literal { opt_prop_declaration }

@R opt_prop_declaration0 :: opt_prop_declaration >>
@R opt_prop_declaration1 :: opt_prop_declaration >> prop_declaration_list

@R prop_declaration_list1 :: prop_declaration_list >> prop_declaration
@R prop_declaration_list2 :: prop_declaration_list >> prop_declaration , prop_declaration_list
@R prop_declaration :: prop_declaration >>
    primary : identifier opt_default

@R opt_default0 :: opt_default >>
@R opt_default1 :: opt_default >> DEFAULT expression
"""

#############################################################################
# CREATE INDEX
#############################################################################

create_index_statement = """
@R statement_create3 :: statement >>
    CREATE INDEX identifier ON reference_list HAVING index_list opt_restrict opt_recursive ; 

## Index List -- Tested
@R index_list :: index_list >> index_declaration_list 

@R index_declaration_list1 :: index_declaration_list >> index_declaration
@R index_declaration_list2 :: index_declaration_list >> index_declaration , index_declaration_list
@R index_declaration :: index_declaration >> identifier : identifier 
"""

#############################################################################
# CREATE SEQUENCE
#
# Sequences can be used in object creation (INSERT) to automatically create
# an unique id for the object. Implementation could be achieved by using
# generators (see Python version 2.2)
#############################################################################

create_sequence_statement = """
@R statement_create4 :: statement >>
    CREATE SEQUENCE identifier opt_increment opt_minvalue opt_maxvalue opt_start ;

@R opt_increment0 :: opt_increment >>
@R opt_increment1 :: opt_increment >> INCREMENT integer

@R opt_minvalue0 :: opt_minvalue >>
@R opt_minvalue1 :: opt_minvalue >> MINVALUE integer

@R opt_maxvalue0 :: opt_maxvalue >>
@R opt_maxvalue1 :: opt_maxvalue >> MAXVALUE integer

@R opt_start0 :: opt_start >>
@R opt_start1 :: opt_start >> START integer
"""

#############################################################################
# INSERT
#############################################################################

insert_statement = """
@R statement_insert1 :: statement >>
    INSERT primary AS identifier INTO primary
    opt_assignment ;
@R statement_insert2 :: statement >>
    INSERT primary AS string_literal INTO primary
    opt_assignment ;
@R statement_insert3 :: statement >>
    INSERT primary AS call INTO primary
    opt_assignment ;
"""

#############################################################################
# UPDATE
#############################################################################

#XXX What's this IN about??? Maybe WHERE is more appropiate
update_statement = """
@R statement_update :: statement >>
    UPDATE reference_list opt_in SET assignment_list opt_restrict opt_recursive ;

## Describes the IN option -- Tested
@R opt_in0 :: opt_in >>
@R opt_in1 :: opt_in >> IN reference_list
"""

#############################################################################
# DELETE
#############################################################################

delete_statement = """
@R statement_delete :: statement >>
    DELETE selection opt_restrict opt_recursive ;
"""

#############################################################################
# SELECT
#############################################################################

#XXX selections should be extended to function results to

basic_statement = """
@R statement_select :: statement >>
    SELECT selection opt_of_type opt_restrict opt_orderby opt_limit opt_recursive ;

## How does a selection list look like? -- Tested
@R selection1 :: selection >> selection_item_list opt_from
@R selection2 :: selection >> ast FROM reference_list 

@R selection_item_list1 :: selection_item_list >> selection_item
@R selection_item_list2 :: selection_item_list >>
    selection_item , selection_item_list

@R selection_item :: selection_item >> primary opt_alias

@R opt_alias0 :: opt_alias >>
@R opt_alias1 :: opt_alias >> AS identifier

## Define what references can be looked into -- Tested
@R opt_from0 :: opt_from >>
@R opt_from1 :: opt_from >> FROM reference_list
"""

type_option = """
## Describe how the TYPE OF should work -- Tested
@R opt_of_type0 :: opt_of_type >> 
@R opt_of_type1 :: opt_of_type >> OF TYPE return_type

@R return_type1 :: return_type >> BRAIN primary
@R return_type2 :: return_type >> OBJECT
"""

# ORDER BY option
order_option = """
@R opt_orderby0 :: opt_orderby >>
@R opt_orderby1 :: opt_orderby >> ORDER BY order_list

@R order_list0 :: order_list >> order_item
@R order_list1 :: order_list >> order_item , order_list

@R order_item :: order_item >> identifier opt_order_dir

@R opt_order_dir0 :: opt_order_dir >>
@R opt_order_dir1 :: opt_order_dir >> ASC
@R opt_order_dir2 :: opt_order_dir >> DESC
"""

# LIMIT option
limit_option = """
@R opt_limit0 :: opt_limit >> 
@R opt_limit1 :: opt_limit >> LIMIT integer opt_offset
@R opt_offset0 :: opt_offset >> 
@R opt_offset1 :: opt_offset >> OFFSET integer
"""

select_statement = basic_statement + type_option + order_option + limit_option

#############################################################################
# ALTER
#############################################################################

alter_statement = """
@R statement_alter :: statement >>
    ALTER CLASS primary alter_statement ; 

## Since there are three different ways to alter an existing class, we
## define three statements handling the cases 
@R alter_statement1 :: alter_statement >>
    ADD object_classification prop_declaration
@R alter_statement2 :: alter_statement >>
    RENAME primary TO primary
@R alter_statement3 :: alter_statement >>
    DELETE reference_list

## Define the possible object classifications for object
## manipulation -- Tested
@R object_classification1 :: object_classification >> PRODUCT
@R object_classification2 :: object_classification >> CLASS
@R object_classification3 :: object_classification >> METHOD
@R object_classification4 :: object_classification >> PROPERTY
"""

############################################################################
#reference lists
#
# possible form of references:
#
############################################################################

references = """
@R reference_list1 :: reference_list >> primary
@R reference_list2 :: reference_list >> primary , reference_list
"""

############################################################################
# Assignments
############################################################################

#XXX unify assignment rules >> reference operator expression
assignments = """
## an optional assignment list, used in CREATE CLASS
@R opt_assignment0 :: opt_assignment >>
@R opt_assignment1 :: opt_assignment >> SET assignment_list

## Now we define the true assignment list -- Tested
@R assignment_list1 :: assignment_list >> assignment
@R assignment_list2 :: assignment_list >> assignment , assignment_list

## Let's see what type of assigments are possible -- Tested

## assignment to identifier
@R assignment1 :: assignment >> keyword_item

## assignment to attribute_ref
@R assignment2 :: assignment >> primary . keyword_item

## assignment to subscription
@R assignment3 :: assignment >> subscription assign expression

## assignment to zopish var
@R assignment4 :: assignment >> zoperef assign expression

## Convenient C-style assignments
@R assignment5 :: assignment >> primary cassign_op expression

@R cassign_op1 :: cassign_op >> plusequal
@R cassign_op2 :: cassign_op >> minusequal
@R cassign_op3 :: cassign_op >> mulequal
@R cassign_op4 :: cassign_op >> divequal
@R cassign_op5 :: cassign_op >> modequal

"""

##########################################################################
# WHERE clause
##########################################################################

clause = """
## Oh, that one is important; it allows for the WHERE clause in SELECT -- Tested
@R opt_restrict0 :: opt_restrict >>
@R opt_restrict1 :: opt_restrict >> WHERE expression
"""

#XXX eliminate this string by splitting to more meaningful sections
oqlgrammar = """
## We should always know, whether we doing something recursivly -- Tested
@R opt_recursive0 :: opt_recursive >>
@R opt_recursive1 :: opt_recursive >> RECURSIVE

## *** Expressions from Python Language Reference ***

## Primaries
## From Python Language Reference, Chapter 5.3 -- Tested

@R primary1 :: primary >> attribute_ref
@R primary2 :: primary >> subscription
@R primary3 :: primary >> call


## Primaries
## From Python Language Reference, Chapter 5.3.1 -- Tested
@R attribute_ref1 :: attribute_ref >> identifier
@R attribute_ref2 :: attribute_ref >> primary . identifier
@R attribute_ref3 :: attribute_ref >> zoperef
@R attribute_ref4 :: attribute_ref >> primary . zoperef

## Subscriptions
## From Python Language Reference, Chapter 5.3.2 -- Tested
@R subscription :: subscription >> primary [ expression_list ]

## Slicings
## From Python Language Reference, Chapter 5.3.3
## XXX Slicing currently ignored


## Call
## From Python Language Reference, Chapter 5.3.4 -- Error
@R call :: call >> primary ( opt_argument_list )

@R opt_argument_list0 :: opt_argument_list >>
@R opt_argument_list1 :: opt_argument_list >> argument_list 

@R argument_list1 :: argument_list >> positional_arguments opt_keyword_arguments
@R argument_list2 :: argument_list >> keyword_item_list

@R opt_keyword_arguments1 :: opt_keyword_arguments >>
## XXX !!! Required ',' throws shift/reduce error !!!
## @R opt_keyword_arguments2 :: opt_keyword_arguments >> , keyword_arguments
@R opt_keyword_arguments2 :: opt_keyword_arguments >> keyword_arguments

@R positional_arguments :: positional_arguments >> expression_list

@R keyword_arguments :: keyword_arguments >> keyword_item_list

@R keyword_item_list1 :: keyword_item_list >> keyword_item
@R keyword_item_list2 :: keyword_item_list >> keyword_item , keyword_item_list

@R keyword_item :: keyword_item >> identifier assign expression


## Operators (level 1)
## From Python Language Reference, Chapter 5.4 -- Tested
@R power1 :: power >> atom
@R power2 :: power >> atom dast u_expr 

##XXX changed primary to atom above

## Unary operations (level 2)
## From Python Language Reference, Chapter 5.5 -- Tested
@R u_expr1 :: u_expr >> power
@R u_expr2 :: u_expr >> minus u_expr
@R u_expr3 :: u_expr >> plus u_expr
@R u_expr4 :: u_expr >> tilde u_expr


## Binary operations (level 3)
## From Python Language Reference, Chapter 5.6 -- Tested
@R m_expr1 :: m_expr >> u_expr
@R m_expr2 :: m_expr >> m_expr ast u_expr
@R m_expr3 :: m_expr >> m_expr slash u_expr
@R m_expr4 :: m_expr >> m_expr percent u_expr

@R a_expr1 :: a_expr >> m_expr
@R a_expr2 :: a_expr >> a_expr plus m_expr
@R a_expr3 :: a_expr >> a_expr minus m_expr


## Shift expressions (level 4)
## From Python Language Reference, Chapter 5.7 -- Tested
@R shift_expr1 :: shift_expr >> a_expr
@R shift_expr2 :: shift_expr >> shift_expr dgreater a_expr 
@R shift_expr3 :: shift_expr >> shift_expr dless a_expr 


## Binary bit operations (level 5)
## From Python Language Reference, Chapter 5.8 -- Tested
@R and_expr1 :: and_expr >> shift_expr
@R and_expr2 :: and_expr >> and_expr amper shift_expr

@R xor_expr1 :: xor_expr >> and_expr
@R xor_expr2 :: xor_expr >> xor_expr circumflex and_expr

@R or_expr1  :: or_expr  >> xor_expr
@R or_expr2  :: or_expr  >> or_expr pipe xor_expr


## Comparisons (level 6)
## From Python Language Reference, Chapter 5.9 -- Tested
@R comparison1 :: comparison >> or_expr 
@R comparison2 :: comparison >> or_expr comparison_operator or_expr

@R comparison_operator1 :: comparison_operator >> less 
@R comparison_operator2 :: comparison_operator >> greater
@R comparison_operator3 :: comparison_operator >> equal 
@R comparison_operator4 :: comparison_operator >> gtequal 
@R comparison_operator5 :: comparison_operator >> ltequal 
@R comparison_operator6 :: comparison_operator >> nequal 
@R comparison_operator7 :: comparison_operator >> IS
@R comparison_operator8 :: comparison_operator >> IS NOT
@R comparison_operator9 :: comparison_operator >> IN
@R comparison_operator10 :: comparison_operator >> NOT IN


## Boolean Operations (level 6)
## From Python Language Reference, Chapter 5.10 -- Tested
@R expression1 :: expression >> or_test

@R or_test1 :: or_test >> and_test
@R or_test2 :: or_test >> or_test OR and_test

@R and_test1 :: and_test >> not_test
@R and_test2 :: and_test >> and_test AND not_test

@R not_test1 :: not_test >> comparison
@R not_test2 :: not_test >> NOT not_test

## Expression lists
## From Python Language Reference, Chapter 5.10 -- Tested
@R expression_list1 :: expression_list >> expression
@R expression_list2 :: expression_list >> expression , expression_list
"""

##########################################################################
# Atoms
##########################################################################

atoms = """
## Atoms - Things which stand for their own, building blocks of expressions
## From Python Language Reference, Chapter 5.2 -- Tested
@R atom1 :: atom >> primary
@R atom2 :: atom >> literal
@R atom3 :: atom >> enclosure

@R enclosure1 :: enclosure >> parenth_form
@R enclosure2 :: enclosure >> list_display
@R enclosure3 :: enclosure >> dict_display
@R enclosure4 :: enclosure >> string_conversion

## Literal Tokens --> see oqlgen.py for definition
## From Python Language Reference, Chapter 5.2.2 -- Tested
@R literal1 :: literal >> string_literal
@R literal2 :: literal >> integer
@R literal3 :: literal >> longinteger
@R literal4 :: literal >> floatnumber
@R literal5 :: literal >> datetime


## Enclosures
## From Python Language Reference, Chapter 5.2.3 and 5.2.4 -- Tested
@R parenth_form :: parenth_form >> ( expression_list )
@R list_display :: list_display >> [ expression_list ]


## Dictionary Display
## From Python Language Reference, Chapter 5.2.5 -- Tested
@R dict_display :: dict_display >> { key_datum_list }

@R key_datum_list1 :: key_datum_list >> key_datum  
@R key_datum_list2 :: key_datum_list >> key_datum , key_datum_list
@R key_datum :: key_datum >> expression : expression

## From Python Language Reference, Chapter 5.2.5 -- Tested
@R string_conversion :: string_conversion >> ` expression `
"""

oqlrules = string.join([script,
                        oqlgrammar,
                        atoms,
                        create_prod_statement,
                        create_class_statement,
                        create_index_statement,
                        create_sequence_statement,
                        insert_statement,
                        update_statement,
                        delete_statement,
                        select_statement,
                        alter_statement,
                        references,
                        assignments,
                        clause,
                        propertysheets,
                        ]
                       )

### list of all nonterminal symbols
nonterms = """
statement_list statement product_type opt_option opt_inherit opt_propertysheet_declaration
opt_assignment opt_from opt_in opt_restrict opt_recursive assignment_list
selection selection_list selection_item opt_of_type return_type propertysheet_list
assignment_list assignment assign_op expression boolean_expression propertysheet propertysheet_list
opt_prop_declaration prop_declaration_list prop_declaration opt_default atom literal
enclosure parenth_form opt_orderby opt_order_dir order_item order_list opt_limit opt_offset
alter_statement object_classification list_display dict_display string_conversion
expression_list key_datum_list opt_comma key_datum primary attribute_ref reference_list
reference_separator subscription call opt_argument_list argument_list positional_arguments
opt_keyword_arguments keyword_arguments keyword_item_list keyword_item power u_expr m_expr
a_expr shift_expr and_expr xor_expr or_expr opt_comp_expr comparison comparison_operator
or_test lambda_form and_test not_test expression_list reference index_declaration_list
index_declaration index_list opt_alias selection_item selection selection_item_list cassign_op oql_script
opt_increment opt_minvalue opt_maxvalue opt_start
"""

### keyword list
keywords = """
ADD ALTER AS ASC BRAIN BY CLASS CREATE DECLARE DEFAULT
DELETE DESC FROM HAVING IN INCREMENT INDEX INHERIT INSERT INTO
LIMIT MAXVALUE METHOD MINVALUE OBJECT OF OFFSET ON OPTION
ORDER PROPERTY PRODUCT PYTHON RECURSIVE RENAME SELECT
SEQUENCE SET START TO TYPE UPDATE WHERE ZCLASS NOT IS
AND OR XOR LAMBDA
"""

### punctuation chars
puncts = """(){}[],.:`;"""



