# This code is copyrighted by Stephan Richter and
# Thomas Foerster in September, 2001.
#
# written by Stephan Richter
# Last updated: October 13, 2001

try:
    MARSHALFILE = INSTANCE_HOME + "/var/oql.mar"
except: # INSTANCE_HOME not defined
    MARSHALFILE = "oql.mar"
        
class ZOQLParser:
    """A parser for ZOQL"""
    
    def __init__(self, marshalfile=MARSHALFILE, bind=1):

        try:
            self._p = self.reloadGrammar(marshalfile)
        except:
            self._p = self.buildGrammar(marshalfile)


        # by default bind semantics to grammar 
        if bind:
            self.bindGrammar()

            
    def parse(self, text, context={}):
        """Parse a ZOQL Script. If no semantics are bound to the parser,
        only syntactic checking is performed.
        """
        return self._p.DoParse(text, context)

        
    def reloadGrammar(self, marshalfile):
        """Reload a marshalled grammar"""

        from kwParsing import kjParser
        from ZOQLLexer import DeclareTerminals
        
        infile = open(marshalfile, "rb")
        OQLG = kjParser.UnMarshalGram(infile)
        infile.close()
        DeclareTerminals(OQLG)

        return OQLG


    def buildGrammar(self, marshalfile):
        """Compile Grammar and marshal to file"""

        from kwParsing import kjParseBuild
        from ZOQLLexer import DeclareComments, DeclareTerminals
        from ZOQLGrammar import oqlrules, nonterms, keywords, puncts

        # Assemble Parser
        OQLG = kjParseBuild.NullCGrammar()
        OQLG.SetCaseSensitivity(0)

        # Lexer related
        DeclareTerminals(OQLG)
        DeclareComments(OQLG)

        # Grammar related
        OQLG.Keywords(keywords)
        OQLG.punct(puncts)
        OQLG.Nonterms(nonterms)
        OQLG.Declarerules(oqlrules)

        print "compiling Grammar..."
        OQLG.Compile()

        print "dumping to", marshalfile
        outfile = open(marshalfile, "wb")
        OQLG.MarshalDump(outfile)
        outfile.close()

        return OQLG


    def bindGrammar(self):
        """Bind semantic functions to parser"""

        import ZOQLSemantics
        
        ZOQLSemantics.BindRules(self._p)


    

    
    

    
