# This code is copyrighted by Stephan Richter and
# Thomas Foerster in September, 2001.
#
# written by Stephan Richter
# Last updated: October 4, 2001

# *** this file implements the semantics of the ZOQL ***
#
# example:
# def example_method( list, context):
#     method body
#
# list - list of parsed elements of the rule,
# e.g. ['CREATE', 'PRODUCT', 'Test'...
# nonterminals come out as a list, rather than relying on the nesting
# level you should write another semantic function to retrieve the item
# itself (or use one of the helpers --> item, item1 ...)
# context - global parse context (variables etc.), if any ;-)

import string, sys
import Products
import Operators, ParsedObjects
from ZOQLExceptions import *

# global method binding dictionary
# key is a semantic method name
# value is list with syntax rules bound to that method 
bindings = {}


def flag_true(list, context):
    return 1

bindings['flag_true'] = ["opt_recursive1"]


def flag_false(list, context):
    return 0

bindings['flag_false'] = ["opt_recursive0", "opt_offset0"]


def list(list, context):
    return list

bindings['list'] = ["alter_statement1", "alter_statement2", "alter_statement3",]


def item(list, context):
    return list[0]

bindings['item'] = ["product_type1", "product_type2", "opt_recursive1", 
                    "opt_prop_declaration1", 
                    "opt_order_dir1", "opt_order_dir2", 
                    "atom1", "atom2", "atom3", "enclosure1",
                    "enclosure2", "enclosure3", "enclosure4",
                    "key_datum_list1", "literal1", "literal2",
                    "literal3", "literal4", "literal5",
                    "primary1", "primary2",
                    "primary3", "opt_argument_list1",
                    "power1", "u_expr1", "m_expr1",
                    "a_expr1", "expression_list1", "not_test1",
                    "and_test1", "or_test1", "or_expr1",
                    "and_expr1", "xor_expr1", "shift_expr1",
                    "comparison1",
                    "comparison_operator1", "comparison_operator2",
                    "comparison_operator3", "comparison_operator4",
                    "comparison_operator5", "comparison_operator6",
                    "comparison_operator7", "comparison_operator8", 
                    "comparison_operator9", "comparison_operator10",
                    "cassign_op1", "cassign_op2", "cassign_op3",
                    "cassign_op4", "cassign_op5", 
                    "attribute_ref1","attribute_ref3",
                    "assignment1","expression1",
                    "index_list",
                    "object_classification1", "object_classification2", 
                    "object_classification3", "object_classification4", 
                    ]


def item1(list, context):
    return list[1]

bindings['item1'] = ["opt_inherit1", "opt_propertysheet_declaration1",
                     "opt_offset1",
                     "opt_assignment1", "opt_from1",
                     "opt_in1", "opt_default1",
                     "list_display", "dict_display", 
                     "opt_alias1", "u_expr3",
                     "opt_increment1",
                     "opt_minvalue1", "opt_maxvalue1",
                     "opt_start1",
                     ]


def item2(list, context):
    return list[2]

bindings['item2'] = ["opt_orderby1", "opt_of_type1"]


def item3(list, context):
    return list[3]


def empty(list, context):
    return None

bindings['empty'] = ["opt_restrict0",
                     "opt_recursive0", "opt_orderby0",
                     "opt_default0", "opt_argument_list0",
                     "opt_of_type0", "opt_in0", "opt_alias0",
                     "opt_limit0", "opt_order_dir0",
                     "statement_list1", "statement_list2",
                     "opt_option0", "opt_increment0",
                     "opt_minvalue0", "opt_maxvalue0",
                     "opt_start0",
                     ]

def empty_list(list, context):
    return []

bindings['empty_list'] = ["opt_inherit0", "opt_from0", 
                          "opt_prop_declaration0",
                          "opt_assignment0",
                          "opt_propertysheet_declaration0",
                          ]


def one_element_list(list, context):
    return [list[0]]

bindings['one_element_list'] = ["propertysheet_list1",
                                "index_declaration_list1",
                                "prop_declaration_list1",
                                "order_list0", "reference_list1",
                                "keyword_item_list1", "assignment_list1",
                                "selection_item_list1",
                                ]


def insert_element(list, context):
    """ Prepend the first element to the list in the third element
    Assumes list to be: [element, ',', element_list]
    """
    if type(list[2]) == type([]):
        result = list[2]
    else:
        result = [list[2]]
        
    result.insert(0, list[0])
    return result

bindings['insert_element'] = ["assignment_list2", "propertysheet_list2",
                              "prop_declaration_list2", "order_list1", 
                              "expression_list2", "index_declaration_list2",
                              "reference_list2", "keyword_item_list2",
                              "selection_item_list2",
                              ]


def append_element(list, context):
    """ Append the third list element to the first element list
    Assumes list to be: [element_list, ',', element]
    """
    if type(list[2]) == type([]):
        result = list[2]
    else:
        result = [list[2]]
    
    result.append(list[2])
    return result


def toString(list, context):
    result = str(list[1])
    return result

bindings['toString'] = ["string_conversion"]


def create_dictionary(list, context):
    """ Convert a [key, ':', value] list to dictionary """ 
    result = {}
    result[list[0]] = list[2]
    return result

bindings['create_dictionary'] = ["key_datum"]


def create_tuple(list, context):
    return tuple(list[1])

bindings['create_tuple'] = ["parenth_form"]


# *** Starting Parsed Objects ***

def get_attribute_reference(list, context):
    parent = list[0]
    child = list[2]
    
    path = parent.get_reference() + '.' + child.get_reference()
    return ParsedObjects.Reference(path)

bindings['get_attribute_reference'] = ["attribute_ref2", "attribute_ref4"]

def property_rename(list, context):
    return ParsedObjects.PropertyRename(list[0], list[2])


def return_type(list, context):
    type = list[0]
    if len(list) > 1:
        klass = list[1]
    else:
        klass = None
    return ParsedObjects.ReturnType(type, klass)

bindings['return_type'] = ["return_type1", "return_type2"]


def class_creation_options(list, context):
    "Wrap class creation options in an object"
    list = list[1]
    creation_options = ParsedObjects.ClassCreationOptions()
    
    for assignment in list:
        creation_options.addOption(assignment)

    return creation_options

bindings['class_creation_options'] = ["opt_option1"]

# *** Starting Arithmetic Operators ***

def add(list, context):
    return Operators.Add(list[0], list[2])

bindings['add'] = ["key_datum_list2", "a_expr2"]
    

def minus(list, context):
    return Operators.Subtract(list[0], list[2])

bindings['minus'] = ["a_expr3"]


def mul(list, context):
    return Operators.Multiply(list[0], list[2])

bindings['mul'] = ["m_expr2"]


def power(list, context):
    return Operators.Power(list[0], list[2])

bindings['power'] = ["power2"]


def div(list, context):
    return Operators.Divide(list[0], list[2])

bindings['div'] = ["m_expr3"]


def mod(list, context):
    return Operators.Modular(list[0], list[2])

bindings['mod'] = ["m_expr4"]



# *** Starting Boolean Operators ***

def not_func(list, context):
    return (1, Operators.Not(None, list[1]))

bindings['not_func'] = ["not_test2"]
    
    
def and_func(list, context):
    return (1, Operators.And(list[0], list[2]))

bindings['and_func'] = ["and_test2"]


def or_func(list, context):
    return (1, Operators.Or(list[0], list[2]))

bindings['or_func'] = ["or_test2"]


def xor_func(list, context):
    return (1, Operators.Xor(list[0], list[2]))

bindings['xor_func'] = ["xor_expr2"]


# *** Starting Bit Operations ***
def bit_and_func(list, context):
    raise NotImplemented

bindings['bit_and_func'] = ["and_expr2"]


def bit_or_func(list, context):
    raise NotImplemented

bindings['bit_or_func'] = ["or_expr2"]


def bit_xor_func(list, context):
    raise NotImplemented

bindings['xor_func'] = ["xor_expr2"]


def inversion_func(list, context):
    raise NotImplemented

bindings['inversion_func'] = ["u_expr4"]


# *** Starting unary operations ***
def negation_func(list, context):
    return Operators.Multiply(-1, list[1])

bindings['negation_func'] = ["u_expr2"]


# *** Starting shift operations ***
def shift_right(list, context):
    raise NotImplemented

bindings['shift_right'] = ["shift_expr2"]


def shift_left(list, context):
    raise NotImplemented

bindings['shift_left'] = ["shift_expr3"]


# *** Starting PropertySheet Handlers ***

def get_propertysheet(list, context):
    return ParsedObjects.PropertySheet(list[0].get_reference(), list[2])

bindings['get_propertysheet'] = ["propertysheet1", "propertysheet2",]


def get_property(list, context):
    return ParsedObjects.Property(list[0].get_reference(),
                                  list[2].get_reference(),
                                  list[3])

bindings['get_property'] = ["prop_declaration"]


# *** Starting Index Handlers ***

def get_index_declaration(list, context):
    ''' '''
    index = {}
    index['name'] = list[0].get_reference()
    index['type'] = list[2].get_reference()
    return index

bindings['get_index_declaration'] = ["index_declaration"]


# *** Starting Limit Handlers ***

def get_limit(list, context):
    ''' '''
    limit = {}
    limit['size'] = list[1]
    limit['start'] = list[2]
    if not limit['start']:
        limit['start'] = 0
    return limit

bindings['get_limit'] = ["opt_limit1"]


# *** Starting Limit Handlers ***

def get_order_item(list, context):
    ''' '''
    order = {}
    order['property'] = list[0]
    order['direction'] = list[1]
    if not order['direction']:
        order['direction'] = 'ASC'
    return order

bindings['get_order_item'] = ["order_item"]

def do_selection1(list, context):
    return (list[0], list[1])

bindings['do_selection1'] = ["selection1"]

def do_selection2(list,context):
    return (None, list[2])

bindings['do_selection2'] = ["selection2"]

def do_selection_item(list, context):
    item = list[0]
    alias = list[1] # what are the semantics of "DTMLDocument AS Foo" ???

    return item

bindings['do_selection_item'] = ["selection_item"]

# *** Starting General Handlers ***


def itemlist(list, context):
    if list[1] is not None:
        result = [list[0]]
    else:
        result = list[0]

    return result

# bindings['itemlist'] = [""]


# *** Start of Comparison Operators ***

def comparison(list, context):
    
    lhs = list[0]
    symbol = list[1]    
    rhs = list[2] 
    
    if symbol == '==':
        return Operators.Equal(lhs, rhs)
    elif symbol == '!=' or symbol == '<>':
        return Operators.NotEqual(lhs, rhs)
    elif symbol == '<=':
        return Operators.LessThanEqual(lhs, rhs)
    elif symbol == '>=':
        return Operators.GreaterThanEqual(lhs, rhs)
    elif symbol == '<':
        return Operators.LessThan(lhs, rhs)
    elif symbol == '>':
        return Operators.GreaterThan(lhs, rhs)
    elif string.lower(symbol) == 'in':
        return Operators.In(lhs, rhs)
    elif string.lower(symbol) == 'is':
        return Operators.In(lhs, rhs)
    else:
        raise UnknownComparisonOperator(symbol)

bindings['comparison'] = ["comparison2"]


# *** Start of Declarative (Assignment) Operators ***

def set_equal(list, context):
    return Operators.SetEqual(list[0], list[2])

bindings['set_equal'] = ["keyword_item", "assignment3",
                         "assignment4", "assignment5"]

def tricky_equal(list, context):
    parent = list[0].get_reference()
    child = list[2].lhs.get_reference()
    rhs = list[2].rhs

    return Operators.SetEqual(ParsedObjects.Reference(parent+'.'+child),
                              rhs)

bindings['tricky_equal'] = ["assignment2"]

def plus_equal(list, context):
    return Operators.PlusEqual(list[0], list[2])

def minus_equal(list, context):
    return Operators.MinusEqual(list[0], list[2])

def mul_equal(list, context):
    return Operators.MulEqual(list[0], list[2])

def div_equal(list, context):
    return Operators.DivEqual(list[0], list[2])

def mod_equal(list, context):
    return Operators.ModEqual(list[0], list[2])


# *** Start of Miscelleneous Handlers ***

# def expression(list, context):
#     return list[0][1]

# bindings['expression'] = ["expression1"]


def itemaslist(list, context):
    return [list[0]]

# bindings['itemaslist'] = []


def item1aslist(list, context):
    return [list[1]]

bindings['item1aslist'] = ["opt_restrict1"]


def function_call(list, context):
    """A python function call. Returns result of function call"""

    funobject = list[0]
    arguments = list[2]
    
    raise NotImplemented, "Function Calls not implemented yet"

bindings['function_call'] = ["call"]


# *** Meta Type List Handler ***

def get_meta_type_list(raw_meta_types):
    raw_meta_types = raw_meta_types
    meta_types = []
    for meta_type in raw_meta_types:
        if type(meta_type) in (type(()), type([])):
            meta_types += list(meta_type)
        else:
            meta_types.append(meta_type)
    return meta_types


# *** Beginning of Statement Handlers ***

def create_product(list, context):
    """Create a new product"""

    product_type = string.lower(list[4])
    if product_type == 'zclass':
        st = ParsedObjects.ZClassProductCreation(list[2].get_reference())
    elif product_type == 'python':
        st = ParsedObjects.PythonProductCreation(list[2].get_reference())
    else:
        raise "Wrong Product Type"

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'):
        return st(context['base_object'])
    else:
        raise BadContextError()

bindings['create_product'] = ["statement_create1"]


def create_class(list, context):
    """ Create a class in a product """

    python_products = filter(lambda x, p=Products: hasattr(getattr(p, x), 'initialize'), dir(Products))

    # collect data from list
    class_id = list[2].get_reference()
    prod_id = list[4].get_reference()
    attributes = list[5]
    options = list[6]
    propertysheets = list[7]
    raw_bases = list[8]

    # Now, because of the ZClasses' sort of dumb way of referencing base classes,
    # we got to do some cleanup
    bases = []
    for base in raw_bases:
        base = base.get_path()
        base = string.replace(base, '/', '.')
        base = string.split(base, '.')
        base = string.join([string.join(base[:-1], '.'), base[-1]], '/') 
        bases.append(base)

    if prod_id in python_products:
        st = ParsedObjects.PythonClassCreation(prod_id, class_id, attributes,
                                              options, propertysheets, bases)
    else:
        st = ParsedObjects.ZClassCreation(prod_id, class_id, attributes,
                                         options, propertysheets, bases)

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'):
        st(context['base_object'])
    else:
        raise BadContextError()


bindings['create_class'] = ["statement_create2"]


def create_index(list, context):
    """ Create a ZOQL index using ZCatalog """

    name = list[2].get_reference()
    selected_meta_types = list[4]
    index_list = list[6]
    conditions = list[7]
    if conditions:
        conditions = conditions[0]

    recursive = list[8]

    st = ParsedObjects.IndexCreation(name,
                                     selected_meta_types,
                                     index_list,
                                     conditions,
                                     recursive)
    
    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'): 
        return st(context['base_object'])
    else:
        raise BadContextError()

bindings['create_index'] = ["statement_create3"]


def create_sequence(list, context):
    """Create a sequence object, CURRENTLY NOT INCLUDED in grammar"""

    name = list[2]
    increment = list[3]
    minvalue = list[4]
    maxvalue = list[5]
    start = list[6]

    st = ParsedObjects.SequenceCreation(name,
                                        increment,
                                        minvalue,
                                        maxvalue,
                                        start)
    
    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'): 
        return st(context['base_object'])
    else:
        raise BadContextError()

bindings['create_sequence'] = ["statement_create4"]

    
def alter_class(list, context):
    """Alter a class"""

    class_ref = list[2]
    action_specs = list[3]
    action = string.lower(action_specs[0])

    if action == 'add':
        st = ParsedObjects.AddAlteration(class_ref, action_specs[1], action_specs[2])
    elif action == 'rename':
        st = ParsedObjects.RenameAlteration(class_ref, action_specs[1], action_specs[3]) 
    elif action == 'delete':
        st = ParsedObjects.DeleteAlteration(class_ref, action_specs[1])

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'): 
        return st(context['base_object'])
    else:
        raise BadContextError()


bindings['alter_class'] = ["statement_alter"]



def insert_statement(list, context):
    """ Perform an INSERT """

    meta_type = "%s" % list[1].get_reference()
    if type(list[3]) == type(''):
        id = list[3]
    else:
        id = list[3].get_reference()

    #XXX that's dirty, needs cleanup
    if hasattr(list[5], 'get_path'):
        container_path = list[5].get_path()
    else:
        container_path = list[5]

    assignments = list[6]

    st = ParsedObjects.InstanceCreation(meta_type, id, container_path, assignments)

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'):
        return st(context['base_object'])
    else:
        raise BadContextError()


bindings['insert_statement'] = ["statement_insert1", "statement_insert2",
                                "statement_insert3"]



def update_statement(list, context):
    """ Perform an UPDATE """

    selected_meta_types = list[1]
    searched_containers = list[2]
    assignments = list[4]
    conditions = list[5]
    if conditions:
        conditions = conditions[0]

    recursive = list[6]
        
    st = ParsedObjects.InstanceUpdate(selected_meta_types,
                                      searched_containers,
                                      assignments,
                                      conditions,
                                      recursive
                                      )

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'): 
        return st(context['base_object'])
    else:
        raise BadContextError()


bindings['update_statement'] = ["statement_update"]



def delete_statement(list, context):
    meta_types,searched_containers = list[1]
    conditions = list[2]
    if conditions:
        conditions = conditions[0]
    recursive = list[3]

    if context.has_key('use_indexes'):
        use_indexes = context['use_indexes']
    else:
        use_indexes = 1 # use indexes by default

    st = ParsedObjects.Deletion(meta_types,
                                searched_containers,
                                conditions,
                                recursive,
                                use_indexes
                                )

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'): 
        return st(context['base_object'])
    else:
        raise BadContextError()

bindings['delete_statement'] = ["statement_delete"]


def select_statement(list, context):
    meta_types, searched_containers = list[1]
    return_type = list[2]
    conditions = list[3]
    if conditions:
        conditions = conditions[0]
    orderby = list[4]
    limit = list[5]
    recursive = list[6]

    if context.has_key('use_indexes'):
        use_indexes = context['use_indexes']
    else:
        use_indexes = 1 # use indexes by default
        
    st = ParsedObjects.Selection(meta_types,
                                return_type,
                                searched_containers,
                                conditions,
                                orderby,
                                limit,
                                recursive,
                                use_indexes) 

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            return st
        
    if context.has_key('base_object'):
        result = st(context['base_object'])
        context['result'].append(result) #???
    else:
        raise BadContextError()
        

bindings['select_statement'] = ["statement_select"]


def oql_script(list, context):
    """Execute a whole zoql script if in batchmode"""

    if context.has_key('batch_mode'):
        if context['batch_mode']:
            if context.has_key('base_object'):
                for st in list[0]:
                    st(base_object)
            else:
                raise BadContextError()

            #XXX what should be returned

bindings['oql_script'] = ["oql_script1"]

#def unbound(list, context):
#    "Not implemented rules"
#    raise NotImplemented
#
#bindings['unbound'] = ["alter_statement2", "alter_statement3"]

# *** Binding of rules to semantic functions ***

def BindRules(Grammar):
    """ Single place for method bindings """

    ## there are two ways to bind rules to functions
    ## either use the bindings dictionary
    ##     bindings[funcname] = [list_of_rulenames]
    
    for func in bindings.keys():
        for rule in bindings[func]:
            Grammar.Bind(rule, globals()[func])
