import sys, os, string
import unittest

from Products.ZOQLMethod.Parser import ZOQLParser

### This class tests the different forms of DELETE
### The WHERE clause tests are in a different module 'ClauseTest.py'
### add as much test methods as possible

class DeleteTest(unittest.TestCase):

    def setUp(self):
        
        self.parser = ZOQLParser.ZOQLParser(marshalfile="oql.mar", bind=0)
        self.context = {}

    def testDeleteSimple(self):
        """Simple DELETE statement"""

        try:
            self.parser.parse("DELETE AName;", self.context)
        except:
            self.fail("DELETE AName;")

    def testDeleteSimpleZopish(self):
        """Simple DELETE statement with zopish name"""

        try:
            self.parser.parse("DELETE z'A Name';", self.context)
        except:
            self.fail("DELETE z'A Name';")

    def testDeleteSimpleRecursive(self):
        """Simple recursive DELETE statement"""

        try:
            self.parser.parse("DELETE AName RECURSIVE;", self.context)
        except:
            self.fail("DELETE AName RECURSIVE;")
            
    def testDeleteList(self):
        """DELETE statement with reference list"""

        try:
            self.parser.parse("DELETE AName, BName.Path, CName;", self.context)
        except:
            self.fail("DELETE AName, BName.Path, CName;")

    def testDeleteListZopish(self):
        """DELETE statement with zopish reference list"""

        try:
            self.parser.parse("DELETE z'A Name', z'B Name'.Path;", self.context)
        except:
            self.fail("DELETE z'A Name', z'B Name'.Path;")

    def testDeleteFrom(self):
        """Simple DELETE statement with FROM"""

        try:
            self.parser.parse("DELETE AName FROM Container;", self.context)
        except:
            self.fail("DELETE AName FROM Container;")

    def testDeleteFromFunctionCall(self):
        """Simple DELETE statement with dynamic FROM"""

        try:
            self.parser.parse("DELETE AName FROM AFunction();", self.context)
        except:
            self.fail("DELETE AName FROM AFunction();")

    def testDeleteSimpleFunctionCall(self):
        """Simple DELETE statement with dynamic objects to delete"""

        try:
            self.parser.parse("DELETE AFunction() FROM Container;", self.context)
        except:
            self.fail("DELETE AFunction() FROM Container")
            
    def testDeleteSimpleMultiple(self):
        """Multiple DELETE statement"""

        try:
            self.parser.parse("DELETE AName FROM ContainerA, ContainerB.Path, ContainerC;", self.context)
        except:
            self.fail("DELETE AName FROM ContainerA, ContainerB;")

    def testDeleteAll(self):
        """DELETE * statement"""

        try:
            self.parser.parse("DELETE * FROM Container;", self.context)
        except:
            self.fail("DELETE * FROM Container;")

    def testDeleteAllWithoutSource(self):
        """DELETE * statement with no container specified"""

        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "DELETE *;", self.context)

    def testDeleteAllRecursive(self):
        """Simple DELETE statement"""

        # doesn't make much sense, but is no error either ;-)
        try:
            self.parser.parse("DELETE * FROM Container RECURSIVE;", self.context)
        except:
            self.fail("DELETE * FROM Container RECURSIVE;")



