import sys, os, string
import unittest

from Products.ZOQLMethod.Parser import ZOQLParser

### This class tests the different Forms of INSERT
### add as much test methods as possible

class InsertTest(unittest.TestCase):

    def setUp(self):
        
        self.parser = ZOQLParser.ZOQLParser(marshalfile="oql.mar", bind=0)
        self.context = {}


    def testInsertSimple(self):
        """Simple Insert"""
        
        try:
            self.parser.parse("INSERT AClass AS objectID INTO Container;", self.context)
        except:
            self.fail("INSERT AClass AS objectID INTO Container;")

    def testInsertSimplePath(self):
        """Simple INSERT into Path"""
        
        try:
            self.parser.parse("INSERT AClass AS objectID INTO Container.Path;", self.context)
        except:
            self.fail("INSERT AClass AS objectID INTO Container.Path;")

    def testInsertSimpleIdZopish(self):
        """Simple Insert with zopish ID"""
        
        try:
            self.parser.parse("INSERT AClass AS 'an object ID' INTO Container;", self.context)
        except:
            self.fail("INSERT AClass AS 'an object ID' INTO Container;")

    def testInsertSimpleIdFunction(self):
        """Simple Insert with ID from 'Sequence'/Function"""
        
        try:
            self.parser.parse("INSERT AClass AS UserIds.next() INTO Container;", self.context)
        except:
            self.fail("INSERT AClass AS UsersIds.next() INTO Container;")

    def testInsertSimpleIdZopishRef(self):
        """INSERT with zope ref as id --> error"""
        
        # wrong product type
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "INSERT AClass AS z'an object' INTO Container;",
                              self.context
                              )
            
    def testInsertSimpleMultiple(self):
        """Simple Insert into multiple destinations not allowed"""
        
        self.failUnlessRaises(SyntaxError, self.parser.parse,
                              "INSERT AClass AS objectID INTO ContainerA, ContainerB;",
                              self.context
                              )

            
    def testInsertSimpleZopish(self):
        """Simple INSERT zopish class"""
        
        try:
            self.parser.parse("INSERT z'A Class' AS objectID INTO Container;", self.context)
        except:
            self.fail("INSERT z'A Class' AS objectID INTO Container;")

    def testInsertSimplePathZopish(self):
        """Simple INSERT into zopish path"""
        
        try:
            self.parser.parse("INSERT AClass AS objectID INTO z'A Container'.Path;", self.context)
        except:
            self.fail("INSERT AClass AS objectID INTO z'A Container'.Path;")

    def testInsertSimpleAssignment(self):
        """Simple Insert with property assignment"""
        
        try:
            self.parser.parse("INSERT AClass AS objectID INTO Container SET id='AClass';", self.context)
        except:
            self.fail("INSERT AClass AS objectID INTO Container SET id='AClass';")

    def testInsertSimpleAssignmentFullPath(self):
        """Simple Insert with full specified property assignment"""
        
        try:
            self.parser.parse("INSERT AClass AS objectID INTO Container SET Basic.id='AClass';", self.context)
        except:
            self.fail("INSERT AClass AS objectID INTO Container SET Basic.id='AClass';")
            
    def testInsertComplex(self):
        """Complex (Typical) INSERT """
        
        try:
            self.parser.parse("INSERT MyUser AS user1 INTO MyUserFolderInstance SET username = 'User1', password = 'User1', roles = ['Manager', 'Anonymous'];", self.context)
        except:
            self.fail("INSERT MyUser AS user1 INTO MyUserFolderInstance SET username = 'User1', password = 'User1', roles = ['Manager', 'Anonymous'];")


### Example for an Expected Fail Test
##
##     def testCreateProductOthers(self):
##         """Wrong product type"""
        
##         # wrong product type
        
##         self.failUnlessRaises(SyntaxError, self.parser.parse,
##                               "CREATE PRODUCT Test AS OTHER;",
##                               self.context
##                               )
