# This code is copyrighted by Stephan Richter and
# Thomas Foerster in September, 2001.
#
# written by Stephan Richter
# Last updated: October 4, 2001
from Shared.DC.ZRDB.Aqueduct import BaseQuery, parse, default_input_form
from Globals                 import DTMLFile, Persistent
from Acquisition             import Implicit
from AccessControl.Role      import RoleManager
from OFS.SimpleItem          import Item
from DateTime                import DateTime

from Products.ZOQLMethod.Parser import ZOQLParser

import DocumentTemplate
import sys, string


try:
    from AccessControl import getSecurityManager
except ImportError:
    getSecurityManager = None


class ZOQLQuery(DocumentTemplate.HTML):
    '''Template Compiler; can also define new local tags'''
    pass
    

manage_addZOQLMethodForm = DTMLFile('dtml/manage_addZOQLMethodForm', globals())

def manage_addZOQLMethod(self, id, title='', base_folder='/', arguments='',
                           query='', REQUEST=None, submit=None):
    '''Management method to add a ZOQL Method; return HTML'''

    zm = ZOQLMethod(id, title, base_folder, arguments, query)
    self._setObject(id, zm)

    if REQUEST is not None:
        url = REQUEST['URL1']
        if submit == " Add and Edit ":
            url = "%s/%s/manage_main" %(url, id)
        elif submit == " Add and Test ":
            url = "%s/%s/manage_testForm" %(url, id)
        else:
            url = url + '/manage_main'
            
        REQUEST.RESPONSE.redirect(url)



class ZOQLMethod(BaseQuery, Implicit, Persistent, RoleManager, Item):
    '''ZOQL Method contains a certain query template and is able to execute it with
       the help of the ZOQL Connection'''

    meta_type = 'ZOQL Method'
    
    manage_main = DTMLFile('dtml/manage_editZOQLMethodForm', globals())

    manage_options = (
        {'label':'Edit',     'action':'manage_main', 'help':('ZOQLMethod', 'Syntax.dtml')},
        {'label':'Test',     'action':'manage_testForm', 'help':('ZOQLMethod', 'Syntax.dtml')},
        {'label':'Security', 'action':'manage_access'},
        ) + Item.manage_options
        
    __ac_permissions__=(
        ('View management screens', ('manage_tabs','manage_main',),),
        ('Change ZOQL Methods', ('manage_edit',
                                 'manage_testForm','manage_test')),
        ('Use ZOQL Methods',    ('__call__',''), ('Anonymous','Manager')),
        )

    parser = ZOQLParser.ZOQLParser()

    report = DTMLFile('dtml/manage_testResult', globals())
    manage_showTree = DTMLFile('dtml/manage_showTree', globals())

    def __init__(self, id, title = '', base_folder='/', arguments = '', query = ''):
        '''Initialize the object'''
        self.id            = id
        self.title         = title
        self.base_folder   = base_folder
        if self.base_folder == '':
            self.base_folder = '/'
        self.arguments     = arguments
        self._arg = parse(arguments)
        self.query         = query
        

    _size_changes={
        'Bigger': (5,5),
        'Smaller': (-5,-5),
        'Narrower': (0,-5),
        'Wider': (0,5),
        'Taller': (5,0),
        'Shorter': (-5,0),
        }


    def get_size(self):
        ''' '''
        return len(self.query)
    

    def _er(self, title, base_folder, arguments, query,
            submit, sql_pref__cols, sql_pref__rows, REQUEST):
        
        dr, dc = self._size_changes[submit]
        
        rows = max(1,  int(sql_pref__rows) + dr)
        cols = max(40, int(sql_pref__cols) + dc)
        expirationDate = (DateTime('GMT') + 365).rfc822()
        resp = REQUEST['RESPONSE']
        resp.setCookie('sql_pref__rows', str(rows), path = '/', expires = expirationDate)
        resp.setCookie('sql_pref__cols', str(cols), path = '/', expires = expirationDate)
        return self.manage_main(self, REQUEST, title = title, arguments = arguments,
                                base_folder = base_folder, query = query,
                                sql_pref__cols = cols, sql_pref__rows = rows)


    def manage_edit(self, title = '', base_folder='/', arguments = '', query = '',
                    submit = 'Change', sql_pref__cols='50', sql_pref__rows='20',
                    REQUEST=None):
        '''Edit the properties of the object; return HTML'''

        if self._size_changes.has_key(submit):
            return self._er(title, base_folder, arguments, query,
                            submit, sql_pref__cols, sql_pref__rows, REQUEST)

        self.title         = title
        self.base_folder   = base_folder
        if self.base_folder == '':
            self.base_folder = '/'
        self.arguments_src = self.arguments = arguments
        self._arg          = parse(arguments)
        self.query         = query
        
        if REQUEST is not None:
            if submit == 'Change':
                return self.manage_main(self, REQUEST)
            elif submit == 'Change and Test':
                return self.manage_testForm(REQUEST)


    def manage_testForm(self, REQUEST):
        '''Test the query; return HTML'''
        input_src = default_input_form(self.title_or_id(),
                                       self._arg, 'manage_test',
                                       '''<dtml-with "_(management_view='Test')"><dtml-var manage_tabs></dtml-with>''')

        return DocumentTemplate.HTML(input_src)(self, REQUEST, HTTP_REFERER='')


    def manage_test(self, REQUEST):
        '''do the test query; return HTML'''
        src = "Could not render the query template!"
        res = ()
        report = self.report
        t = v = tb = None
        try:
            try:
                src, res = self(REQUEST, tst__=1)
                # res = self(REQUEST)
                if len(res) and type(res[0]) == type([]):
                    res = res[0]
            except:
                t, v, tb = sys.exc_info()
                html = '<strong>Error, <em>%s</em>:</strong><br> %s' % (t,v)
                print t
                print v

                report = DocumentTemplate.HTML(
                    '<dtml-var manage_page_header>'
                    '<dtml-var manage_tabs>\n<hr><pre>%s</pre>\n\n'
                    '<hr><strong>Expression used:</strong><br>\n<pre>\n%s\n</pre>\n<hr>\n'
                    '<dtml-var manage_page_footer>' % (html, src)
                    )

            report = apply(report,(self, REQUEST), {'result' : res, 'source': src})

            if tb is not None:
                self.raise_standardErrorMessage(
                    None, REQUEST, t, v, tb, None, report)

            return report

        finally: tb = None
                

    def execute_xmlrpc(self, zoql):
        ''' '''
        
        def makeDict(object):
            if hasattr(object, 'getSize'):
                size = object.getSize()
            else:
                size = 0
            return {'id': object.getId(),
                    'meta_type': object.meta_type,
                    'title': object.title,
                    'path': object.absolute_url(),
                    'size': size,
                    'mod_time': str(object.bobobase_modification_time().strftime('%Y/%m/%d %H:%M:%S'))}

        try:
            result = self._execute(zoql)
            if result:
                result = map(makeDict, result[0])
            else:
                result = []
            return {'status': 0, 'values': result}
        except SyntaxError, error:
            return {'status': 1, 'values': error}
        except Exception, error:
            return {'status': 2, 'values': error}
        except:
            return {'status': 3, 'values': {'args': ['Unknown Error'],
                                            'msg': 'Unknown Error: This error was caught last minute!'}}

    def _execute(self, zoql):
        ''' '''

        context = {'base_object': self.unrestrictedTraverse(self.base_folder),
                   'result': []}

        self.parser.parse(zoql, context)

        return context['result']


    def __call__(self, REQUEST = None, src__ = 0, tst__ = 0, **kw):
        '''execute the query; return string'''
        
        # get the environment
        if REQUEST is None:
            if kw:
                REQUEST = kw
            else:
                if hasattr(self, 'REQUEST'):
                    REQUEST = self.REQUEST
                else:
                    REQUEST={}
        
        # get the parent of this ZOQL Query
        if hasattr(self, 'aq_parent'):
            parent = self.aq_parent
        else:
            parent = None

        # use our BaseQuery's magic.  :)
        argdata = self._argdata(REQUEST)
        
        # Also need the authenticated user.
        auth_user = REQUEST.get('AUTHENTICATED_USER', None)
        if auth_user is None:
            auth_user = getattr(self, 'REQUEST', None)
            if auth_user is not None:
                try:
                    auth_user = auth_user.get('AUTHENTICATED_USER', None)
                except:
                    auth_user = None
                
        if auth_user is not None:
            if getSecurityManager is None:
                # working in a pre-Zope 2.2.x instance
                from AccessControl.User import verify_watermark
                verify_watermark(auth_user)
                argdata['AUTHENTICATED_USER'] = auth_user
                
        zoql = ZOQLQuery(self.query)
        zoql.cook()                       
        
        # Working with the new security manager (Zope 2.2.x ++)
        security = getSecurityManager()
        security.addContext(self)
        try:
            zoql = apply(zoql, (parent,), argdata)  # apply the template
        finally:
            security.removeContext(self)
        
        # return the rendered source
        if src__:
            return zoql

        # execute the query 
        result = self._execute(zoql)


        # return the rendered source and the result, so that the template does
        # not need to be rendered twice for the test
        if tst__:
            return zoql, result

        if len(result) == 1:
            result = result[0]

        return result


import Globals
Globals.default__class_init__(ZOQLMethod)
