import os, sys, unittest
sys.path.append('..')

try:
    import Zope
except:
    # for correct zope import we must be in zope/lib/python
    sys.path.insert(1, os.getcwd())
    os.chdir('../../..')
    import Zope
    
from Testing import makerequest
from Products.ZOQLMethod.Parser import ZOQLExceptions

class ZOQLDeleteTests(unittest.TestCase):

    def setUp(self):

        #XXX make use of a DemoStorage and ZEO
        self.app=makerequest.makerequest(Zope.app())

        # create a product and a class to play with
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        self.p = self.app.Control_Panel.Products.TestProduct

        #!!! w/o CreateAFactory the meta type will not be found
        # not registered ???
        self.p.manage_addZClass('TestClass','Class for Unittesting',
                                meta_type='TestClass', CreateAFactory=1, zope_object=1)
        self.p.manage_addZClass('TestClassZ','Class for Unittesting',
                                meta_type='Zopish Test Class', CreateAFactory=1, zope_object=1)
        
        # create nested playground
        self.app.manage_addFolder('TestFolder')
        self.app.TestFolder.manage_addFolder('A')
        self.app.TestFolder.manage_addFolder('B')
        self.app.TestFolder.manage_addDTMLMethod('m')
        self.app.TestFolder.manage_addDTMLDocument('index_html')
        self.app.TestFolder.A.manage_addDTMLDocument('index_html')
        

    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


    ########### Tests ##############

    def testSimpleDeleteAll(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='delAll', query='DELETE * FROM <dtml-var folder>;', arguments='folder')

        self.app.delAll(folder='TestFolder')
        result = self.app.TestFolder.objectValues()
        
        self.failUnlessEqual(result, [])

        
    def testRestrictedDelete(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='delType', query='DELETE z"<dtml-var type>" FROM <dtml-var folder>;', arguments='type\nfolder')

        self.app.delType(type='DTML Method', folder='TestFolder')
        result = self.app.TestFolder.objectValues()
        self.failUnlessEqual(result, [self.app.TestFolder.A,
                                      self.app.TestFolder.B,
                                      self.app.TestFolder.index_html])


    def testRestrictedDeleteRecursive(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='delType', query='DELETE z"<dtml-var type>" FROM <dtml-var folder> RECURSIVE;', arguments='type\nfolder')

        self.app.delType(type='DTML Document', folder='TestFolder')
        result = self.app.TestFolder.objectValues()
        self.failUnlessEqual(result, [self.app.TestFolder.A,
                                      self.app.TestFolder.B,
                                      self.app.TestFolder.m])

        self.failUnlessEqual(self.app.TestFolder.A.objectValues(),[])


    def testRestrictedDeleteMultiple(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='DELETE z"DTML Method",z"DTML Document" FROM <dtml-var folder>;', arguments='folder')

        self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(self.app.TestFolder.objectValues(),
                             [self.app.TestFolder.A, self.app.TestFolder.B])

    def testRestrictedDeleteMultipleRecursive(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='DELETE z"DTML Method",z"DTML Document" FROM <dtml-var folder> RECURSIVE;', arguments='folder')

        self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(self.app.TestFolder.objectValues(),
                             [self.app.TestFolder.A, self.app.TestFolder.B])
        self.failUnlessEqual(self.app.TestFolder.A.objectValues(),
                             [])

        
    def testSpecificId(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='DELETE * FROM <dtml-var folder> WHERE id=="index_html";', arguments='folder')

        self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(self.app.TestFolder.objectValues(),
                             [self.app.TestFolder.A,
                              self.app.TestFolder.B,
                              self.app.TestFolder.m])


    def testSpecificIdRecursive(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='DELETE * FROM <dtml-var folder> WHERE id=="index_html" RECURSIVE;', arguments='folder')

        self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(self.app.TestFolder.objectValues(),
                             [self.app.TestFolder.A,
                              self.app.TestFolder.B,
                              self.app.TestFolder.m])
        self.failUnlessEqual(self.app.TestFolder.A.objectValues(),
                             [])
                    

        
if __name__ == '__main__':
    unittest.main()
