import os, sys, unittest
sys.path.append('..')

try:
    import Zope
except:
    # for correct zope import we must be in zope/lib/python
    sys.path.insert(1, os.getcwd())
    os.chdir('../../..')
    import Zope
    
from Testing import makerequest
from Products.ZOQLMethod.Parser import ZOQLExceptions


class ZOQLInsertTests(unittest.TestCase):

    def setUp(self):
        
        #XXX make use of a DemoStorage and ZEO
        self.app=makerequest.makerequest(Zope.app())

        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(id='ZOQLTestMethod', query='')

        # create a product and a class to play with
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        self.p = self.app.Control_Panel.Products.TestProduct

        #!!! w/o CreateAFactory the meta type will not be found
        # not registered ???
        self.p.manage_addZClass('TestClass','Class for Unittesting', meta_type='TestClass',
                                CreateAFactory=1, zope_object=1)
        self.p.TestClass.propertysheets.common.manage_addCommonSheet('Basic','')
        self.sheet = self.p.TestClass.propertysheets.common['Basic']
        self.sheet.manage_addProperty('Title', '', 'string')
        self.sheet.manage_addProperty('Creation', '2000/01/01 0:0:0', 'date')
        
        self.p.manage_addZClass('TestClassZ','Class for Unittesting',
                                meta_type='Zopish Test Class', CreateAFactory=1, zope_object=1)
        
        # create nested playground
        self.app.manage_addFolder('TestFolder')
        self.app.TestFolder.manage_addFolder('A')
        self.app.TestFolder.manage_addFolder('B')

        
    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


    ########### Tests ##############

    def testSimpleInsert(self):
        self.app.ZOQLTestMethod.manage_edit(query='INSERT TestClass AS TestInstance INTO TestFolder;')
        self.app.ZOQLTestMethod()

        assert 'TestInstance' in self.app.TestFolder.objectIds()
        o = self.app.TestFolder.TestInstance

        self.failUnlessEqual(o.meta_type, 'TestClass')

        
    def testSimpleInsertIntoPath(self):
        self.app.ZOQLTestMethod.manage_edit(query='INSERT TestClass AS TestInstance INTO TestFolder.A;')
        self.app.ZOQLTestMethod()

        assert 'TestInstance' in self.app.TestFolder.A.objectIds()
        o = self.app.TestFolder.A.TestInstance

        self.failUnlessEqual(o.meta_type, 'TestClass')


    def testSimpleInsertOtherBase(self):
        self.app.ZOQLTestMethod.manage_edit(base_folder='TestFolder',
                                            query='INSERT TestClass AS TestInstance INTO A;')
        self.app.ZOQLTestMethod()

        assert 'TestInstance' in self.app.TestFolder.A.objectIds()
        o = self.app.TestFolder.A.TestInstance

        self.failUnlessEqual(o.meta_type, 'TestClass')


    def testSimpleZopishInsert(self):
        self.app.ZOQLTestMethod.manage_edit(
            query='INSERT z"Zopish Test Class" AS TestInstance INTO TestFolder;')
        self.app.ZOQLTestMethod()

        assert 'TestInstance' in self.app.TestFolder.objectIds()
        o = self.app.TestFolder.TestInstance

        self.failUnlessEqual(o.meta_type, 'Zopish Test Class')


    def testSimpleInsertWithAssignment(self):
        self.app.ZOQLTestMethod.manage_edit(
            query='INSERT TestClass AS TestInstance INTO TestFolder SET Basic.Title = "<dtml-var title>";',
            arguments="title")
        self.app.ZOQLTestMethod(title='The Title')

        assert 'TestInstance' in self.app.TestFolder.objectIds()
        o = self.app.TestFolder.TestInstance

        self.failUnlessEqual(o.meta_type, 'TestClass')
        self.failUnlessEqual(o.propertysheets.Basic.Title, 'The Title')


        
if __name__ == '__main__':
    unittest.main()

