import os, sys, unittest
sys.path.append('..')

try:
    import Zope
except:
    # for correct zope import we must be in zope/lib/python
    sys.path.insert(1, os.getcwd())
    os.chdir('../../..')
    import Zope
    
from Testing import makerequest
from Products.ZOQLMethod.Parser import ZOQLExceptions

class ZOQLSelectTests(unittest.TestCase):

    def setUp(self):

        #XXX make use of a DemoStorage and ZEO
        self.app=makerequest.makerequest(Zope.app())

        # create a product and a class to play with
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        self.p = self.app.Control_Panel.Products.TestProduct

        #!!! w/o CreateAFactory the meta type will not be found
        # not registered ???
        self.p.manage_addZClass('TestClass','Class for Unittesting', meta_type='TestClass',
                                CreateAFactory=1, zope_object=1)
        self.p.manage_addZClass('TestClassZ','Class for Unittesting', meta_type='Zopish Test Class',
                                CreateAFactory=1, zope_object=1)
        
        # create nested playground
        self.app.manage_addFolder('TestFolder')
        self.app.TestFolder.manage_addFolder('A')
        self.app.TestFolder.manage_addFolder('B')
        self.app.TestFolder.manage_addDTMLMethod('m')
        self.app.TestFolder.manage_addDTMLDocument('index_html')
        self.app.TestFolder.A.manage_addDTMLDocument('index_html')
        
    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


    ########### Tests ##############

    def testSimpleSelect(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='ls', query='SELECT ALL FROM <dtml-var folder>;', arguments='folder')

        result = self.app.ls(folder='TestFolder')
        self.failUnlessEqual(
            result, [self.app.TestFolder.A,self.app.TestFolder.B,self.app.TestFolder.m, self.app.TestFolder.index_html])

        result = self.app.ls(folder='TestFolder.B')
        self.failUnlessEqual(result, [])

        
    def testRestrictedSelect(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT z"DTML Method" FROM <dtml-var folder>;', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [self.app.TestFolder.m])


    def testRestrictedSelectMultiple(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT z"DTML Method",z"DTML Document" FROM <dtml-var folder>;', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [self.app.TestFolder.m,
                                      self.app.TestFolder.index_html])


    def testRestrictedSelectRecursive(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT z"DTML Document" FROM <dtml-var folder> RECURSIVE;', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [self.app.TestFolder.A.index_html,
                                      self.app.TestFolder.index_html]) 


    def testSpecificId(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT * FROM <dtml-var folder> WHERE id=="index_html";', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [self.app.TestFolder.index_html])


    def testSpecificIdAndMetaType(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT Folder FROM <dtml-var folder> WHERE id=="index_html";', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [])


    def testIgnoreUnknownProperties(self):
        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(
            id='lsd', query='SELECT * FROM <dtml-var folder> WHERE foobarblaz == 5;', arguments='folder')

        result = self.app.lsd(folder='TestFolder')
        self.failUnlessEqual(result, [])


if __name__ == '__main__':
    unittest.main()
