import os, sys, unittest
sys.path.append('..')

try:
    import Zope
except:
    # for correct zope import we must be in zope/lib/python
    sys.path.insert(1, os.getcwd())
    os.chdir('../../..')
    import Zope
    
from Testing import makerequest
from Products.ZOQLMethod.Parser import ZOQLExceptions

class ZOQLMethodTests(unittest.TestCase):

    def setUp(self):
        
        #XXX make use of a DemoStorage and ZEO
        self.app=makerequest.makerequest(Zope.app())

        self.app.manage_addProduct['ZOQLMethod'].manage_addZOQLMethod(id='ZOQLTestMethod', query='')
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')

        try:
            self.app.Control_Panel.Products.manage_delObjects(['TestProduct'])
        except:
            pass
                                                             
        
    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


    ########### Tests ##############

    ### Product Creation
        
    def testFixedProductCreation(self):
        """Creation of a ZClass Product"""
        self.app.ZOQLTestMethod.manage_edit(query='create product TestProduct2 as zclass;')
        self.app.ZOQLTestMethod()

        assert 'TestProduct2' in self.app.Control_Panel.Products.objectIds()


    def testVariableProductCreation(self):
        """Test creation with name parameter""" 
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName', query='create product <dtml-var aName> as zclass;')
        self.app.ZOQLTestMethod(aName='TestProduct3')

        assert 'TestProduct3' in self.app.Control_Panel.Products.objectIds()

        self.failUnlessRaises(KeyError, self.app.ZOQLTestMethod)
        self.failUnlessRaises(SyntaxError, self.app.ZOQLTestMethod, aName='A Product')

        
    def testPythonProductCreation(self):
        """Python Product Creation"""
        try:
            os.system('rm -r Products/TestProduct4')
        except:
            pass
        self.app.ZOQLTestMethod.manage_edit(query='create product TestProduct4 as python;')
        self.app.ZOQLTestMethod()

        assert 'TestProduct4' in self.app.Control_Panel.Products.objectIds()


    ### Class Creation

    def testSimpleClassCreation(self):
        """A very basic class creation without options, attributes and properties"""
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        p = self.app.Control_Panel.Products.TestProduct
        
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName', query='create class AClass in TestProduct;')
        self.app.ZOQLTestMethod()

        try:
            klass = p.AClass
        except AttributeError:
            self.fail()

        assert klass.meta_type == 'Z Class'


    def testClassCreationWithOptions(self):
        """A very basic class creation with options"""
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        p = self.app.Control_Panel.Products.TestProduct
        
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName', query='create class AClass in TestProduct OPTION create_factory=1;')
        self.app.ZOQLTestMethod()

        try:
            klass = p.AClass
        except AttributeError:
            self.fail()

        assert klass.meta_type == 'Z Class'
        obs = p.objectIds()
        assert 'AClass_factory' in obs
        assert 'AClass_addForm' in obs
        assert 'AClass_add' in obs
        assert 'AClass_add_permission' in obs


    def testClassCreationWithInherit(self):
        """A very basic class creation with inheritance"""
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        p = self.app.Control_Panel.Products.TestProduct
        
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName', query='create class AClass in TestProduct INHERIT OFS.Folder.Folder;')
        self.app.ZOQLTestMethod()

        try:
            klass = p.AClass
        except AttributeError:
            self.fail()

        assert klass.meta_type == 'Z Class'
        assert len(klass._zbases) == 2 #XXX better check for right classes


    def testClassCreationWithCustomMetaType(self):
        """A very basic class creation with custom meta_type"""
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        p = self.app.Control_Panel.Products.TestProduct
        
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName', query='create class AClass in TestProduct SET meta_type="ACustomClass";')
        self.app.ZOQLTestMethod()

        try:
            klass = p.AClass
        except AttributeError:
            self.fail()

        self.failUnlessEqual(klass.meta_type, 'Z Class')
        self.failUnlessEqual(klass._zclass_.meta_type, 'ACustomClass')


    def testClassCreationWithPropertySheet(self):
        """A very basic class creation with a propertysheet"""
        self.app.Control_Panel.Products.manage_addProduct('TestProduct', 'TestProduct product')
        p = self.app.Control_Panel.Products.TestProduct
        
        self.app.ZOQLTestMethod.manage_edit(
            arguments='aName',
            query='create class AClass in TestProduct DECLARE Basic {title:string DEFAULT "foobar", num:int DEFAULT 42};')
        self.app.ZOQLTestMethod()

        try:
            klass = p.AClass
        except AttributeError:
            self.fail()

        try:
            sheet = klass.propertysheets.common.Basic
        except AttributeError:
            self.fail()

        assert sheet.hasProperty('title')
        assert sheet.hasProperty('num')
        self.failUnlessEqual(sheet.getProperty('title'), 'foobar')
        self.failUnlessEqual(sheet.getProperty('num'), 42)


if __name__ == '__main__':
    unittest.main()
